<?php

ob_start();

/* 
* Class To handle admin end options
* This is the base class to be extended. see example below
*/
class Fetp_Admin {
	
	/* Admin Page Title */
	var $title;
	
	/* Admin page menu title */
	var $menu;
	
	/* User capabilty */
	var $cap;
	
	/* 
	* Admin page slug
	*
	* (admin.php?page=$slug)
	*/
	var $slug;
	
	/* Add the admin page to the admin_menu */
	var $page;
	
	
	/*
	* Post type for the admin page. This is where we hook the metaboxes
	*
	* @var $post_type (string)
	*/
	var $post_type;
	
	/*
	* Class constructor
	*/
	public function __construct( $title = 'Front End Theme Preview Settings', $menu = 'Fetp Options', $cap = 'manage_options', $slug = 'fetp'  ) {
		/* Parameters to add a menu page */
		$this->title = $title;
		$this->menu = $menu;
		$this->cap = $cap;
		$this->slug = $slug;
		$this->post_type = 'toplevel_page_' . $slug;
		
		/* Add the page */
		add_action( 'admin_menu', array( $this, 'add_page' ) );
		
		/* process form submission */
		add_action( 'admin_post_save_fetp_form', array( $this, 'update_options' ) );
		
		/* Default metaboxes */
		add_action( 'add_meta_boxes', array( $this, 'meta_boxes' ) );
	}
	
	/* 
	* Custom Metabox 
	*
	* @param $metabox - (array) parameters to setup a metabox
		- array(
			'id' => 'metabox_id',
			'title' => 'Metabox Title',
			'callback' => array( $this, 'metabox_content_function' ),
			'post_type' => $this->post_type, // hook meta boxes to the plugin admin page
			'context' => 'high' 'advanced' or 'side',
			'priority' => 'high' 'low', 'core' or 'default'
		);
	*/
	public function add_meta_box( $metabox ) {
	global $current_screen;
		if ( empty( $metabox ) ) {
			trigger_error( 
				__( 'The function <code>'. __FUNCTION__ .'</code> was called with an empty array argument' ), headers_sent() || WP_DEBUG ? E_USER_WARNING : E_USER_NOTICE );
		}
		
		/* requires a minimum of 6 values */
		if ( count( $metabox ) < 6 ) {
			trigger_error( __( 'The function <code>'. __FUNCTION__ .'</code> expects a minimum of six keys in the array parameter' ), headers_sent() || WP_DEBUG ? E_USER_WARNING : E_USER_NOTICE );
		}
		
		extract( $metabox );
		
		add_meta_box( $id, $title, $callback, $post_type, $context, $priority );
	}
	
	/**
	 * Adds the custom page.
	 * Adds callbacks to the load-* and admin_footer-* hooks
	*/
	function add_page() {

		/* Add the page */
		$this->page = add_menu_page( $this->title, $this->menu, $this->cap, $this->slug, array( &$this, 'render_page' ), plugins_url( 'img/admin-icon.png', __FILE__ ) );

		/* Add callbacks for this screen only */
		add_action( 'load-' . $this->page, array( $this, 'page_actions' ) );
		
		/* Set help tab on page load */
		add_action( 'load-' . $this->page, array( $this, 'set_help_tab' ) );
	}
	
	/*
	* This calls the add_meta_boxes hooks, adds screen options and enqueues the postbox.js, jquery scripts.
	*/
	function page_actions() {
		do_action( 'add_meta_boxes_' . $this->page, null);
		do_action( 'add_meta_boxes', $this->page, null);
		add_action( 'admin_footer-' . $this->page, array( $this, 'footer_scripts' ), 9 );
		
		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'postbox' );
		
		/* Add scripts on page load */
		add_action( 'admin_print_scripts-' . $this->page, array( $this, 'enqueue_scripts' ) );

		/* 
		* Adjust the screen layout using the filter below. 
		* The plugin has a default of two columns
		*/
		
		/* screen layout filter */
		$screen_layout = apply_filters( 'fetp_admin_screen_layout', array( 'max' => 2, 'default' => 2 ) );
		
		/* Add screen option */
		add_screen_option( 'layout_columns', $screen_layout );
	}
	
	/* Add our scripts */
	public function enqueue_scripts() {
		wp_enqueue_script( 'fetpjs', plugins_url( 'js/fetp-admin.js', __FILE__ ) );
		wp_enqueue_style( 'fetpcss', plugins_url( 'css/fetp-admin.css', __FILE__ ) );
	}

	/**
	 * Prints the jQuery script to initiliase the metaboxes
	 * Called on admin_footer-*
	*/
	function footer_scripts() {
		echo '<script type="text/javascript">postboxes.add_postbox_toggles(pagenow);</script>';
	}	
	
	/**
	 * Renders the admin page
	*/
	function render_page(){
		
		$screen = get_current_screen();
		$columns = $screen->get_option( 'layout_columns', 'max' );
		
		?>
		 <div class="wrap fetp-wrap">
			<br />
			<br />
			<?php $this->options_updated(); ?>
			<form id="fetp_submit" method="post" action="admin-post.php">
				<?php wp_nonce_field( 'fetp-update-options' );			
				/* Used to save closed metaboxes and their order */
				wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce', false );
				wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce', false ); 
				?>
				<input type="hidden" name="action" value="save_fetp_form">	

				<div id="poststuff">
		
					 <div id="post-body" class="metabox-holder columns-<?php echo $columns; ?>">

						  <div id="postbox-container-2" class="postbox-container">
						        <?php do_meta_boxes('','normal',null);  ?>
						        <?php do_meta_boxes('','advanced',null); ?>
						  </div>
						  
						  <div id="postbox-container-1" class="postbox-container">
						        <?php do_meta_boxes('','side',null); ?>
						  </div> 
						  
					 </div>
				
				 </div>

	      		  </form>			

		 </div>
		<?php
	}
	
	/* Add help content */
	public function set_help_tab() {

		$screen = get_current_screen();
		
		$overview = '<h3><strong>Front End Theme Preview By <a href="http://profiles.wordpress.org/harvey_j">Harvey J</a></strong></h3>';
		$overview .= '<p>This plugin allows site visitors to preview your precious themes with the added options to download/buy the theme(s).</p>';
		$overview .= '<p>Tweak the plugin with the options below.</p>';
		
		$screen->add_help_tab(
			array(
				'id' => 'fetp',
				'title' => __( 'Overview', 'fetp' ),
				'content' => $overview
			)
		);
		
		$usage = '<h3><strong>Usage</strong></h3>';
		$usage .= '<p>To get things going, make sure to add the themes you want to make available on the checkbox list below</p>';
		$usage .= '<p>To initiate a preview, load your site like: <code>' . site_url( '/?fet_preview='.get_stylesheet() ) . '</code></p>';
		$usage .= '<p>Site Admins have a bonus option of changing the theme from the front end. See the preview toolbar.</p>';
		$usage .= '<p>There are three widgets which you can place in your sidebar to list the available themes in a dropdown select, another to show the user\'s purchased themes and lastly another widget that displays the plugin\'s paypal shopping cart';
		
		$screen->add_help_tab(
			array(
				'id' => 'fetp_usage',
				'title' => __( 'Usage', 'fetp' ),
				'content' => $usage
			)
		);
		
		$codes = '<h3><strong>Shortcodes</strong></h3>';
		$codes .= '<ol>';
		$codes .= '<li><b>[fetp]</b> &#8212; Displays download/buy and preview links</li>';
		$codes .= '<li><b>[fetpreview theme="$stylesheet"]</b> &#8212; Displays a preview link</li>';
		$codes .= '<li><b>[fetpdownload theme="$stylesheet"]</b> &#8212; Displays a download or a buy link</li>';
		$codes .= '<li><b>[fetpselect]</b> &#8212; Displays a dropdown of available themes which you can place in a sidebar</li>';
		$codes .= '<li><b>[fetplistpremium]</b> &#8212; Displays the logged in user\'s purchased themes</li>';
		$codes .= '</ol>';
		
		$screen->add_help_tab(
			array(
				'id' => 'fetp_codes',
				'title' => __( 'Shortcodes', 'fetp' ),
				'content' => $codes
			)
		);
		
		$hooks = '<h3><strong>Hooks And Filters</strong></h3>';
		$hooks .= '<ol><li><b>Action:</b> To add a custom option to the toolbar like a link or plain text, use the <code>fetp_action</code> hook.</li>';
		$hooks .= '<li><b>Filter:</b> If a premium theme has been deleted, use the <code>fetp_deleted_theme</code> hook to notify the user in the Fetp_Download_Themes Widget</li>';
		$hooks .= '<li><b>Filter:</b> Use <code>fetp_submit_form_metabox</code> to add custom content to the "Update Options" Metabox</li>';
		$hooks .= '<li><b>Filter:</b> Use <code>fetp_admin_screen_layout</code> to adjust the screen layout in <code>add_screen_option()</code></li></ol>';
		
		$screen->add_help_tab(
			array(
				'id' => 'fetp_hooks',
				'title' => __( 'Hooks', 'fetp' ),
				'content' => $hooks
			)
		);
		
		$premium_tab = '<h3><strong>Premium Theme Settings</strong></h3>';
		$premium_tab .= '<p>To add a theme to the premium list, just set it\'s price in the appropriate field below. Easy.</p>';
		$premium_tab .= '<p><strong>NOTE: </strong> Theme prices should have a two decimal place amount.</p>';
		
		$screen->add_help_tab(
			array(
				'id' => 'fetp_premium',
				'title' => __( 'Premium Themes', 'fetp' ),
				'content' => $premium_tab
			)
		);
		
		$screen->set_help_sidebar(
			'<h3><strong>For Questions:</strong></h3>' . 
			'<p><a href="http://wordpress.org/plugins/front-end-theme-preview">Check the Support Forum</a></p>' .
			
			'<h3><strong>For a pat on the back:</strong></h3>' .
			'<p><a href="'. fetp_support_project( true ) .'">Pass by the donation box on your way home</a></p>'
		);
	}
	/* Save form changes */
	function update_options() {
		
		/* Check user capability */
		if ( ! current_user_can( 'manage_options' ) )
			wp_die( 'No chance' );
		
		/* Check referer */
		check_admin_referer( 'fetp-update-options' );
		
		/* 
		* update options
		*
		* first process the themes and their prices (if any) 
		*/
		$posted_themes = ( isset( $_POST['fetp-themes'] ) ) ? $_POST['fetp-themes'] : array();
		
		/* save preview list */
		$fetp_themes = array();
		
		if ( ! empty( $posted_themes ) ) {
			foreach( $posted_themes as $theme ) {
				$fetp_themes[] = $theme;
			}
		}
		
		$premium_themes = array();
		
		/* save price(s) */
		if ( ! empty( $fetp_themes ) ) {
			foreach( $fetp_themes as $stylesheet ) {
				if ( isset( $_POST[ $stylesheet . '_price'] ) )
					$premium_themes[ $stylesheet ] = $_POST[ $stylesheet . '_price'];
			}
		}
		
		/* remove free themes from premium list */
		foreach( $premium_themes as $stylesheet => $price ) {
			if ( empty( $price ) )
				unset( $premium_themes[ $stylesheet ] );
		}
		
		/* Process paypal settings */
		$paypal_settings = array(
			'business' => stripslashes( $_POST['paypal_email'] ),
			'sandbox' => ( ! empty( $_POST['paypal_sandbox'] ) ) ? 1 : 0,
			'currency' => stripslashes( $_POST['paypal_currency'] ),
			'cursymbol' => $_POST['paypal_currencysymbol'],
			'location' => stripslashes( $_POST['paypal_location'] ),
			'ipn_log' => ( ! empty( $_POST['paypal_log'] ) ) ? 1 : 0,
			'subject' => stripslashes( $_POST['paypal_subject'] ),
			'message' => stripslashes( $_POST['paypal_message'] )
		);
		
		// Don't save htaccess in the Database, send it straight to the file
		$htaccess = stripslashes( $_POST['fetp_htaccess'] );
		
		if ( !empty( $htaccess ) && file_get_contents( WP_CONTENT_DIR . '/themes/.htaccess' ) !== $htaccess )
			fwrite( fopen( WP_CONTENT_DIR . '/themes/.htaccess', 'w+' ), $htaccess );
		
		/* 
		* The rest of the form should be sent to the database as is. Someone may have added
		* a custom metabox with input fields and that data should be saved as well. Since themes
		* and paypal settings have already been saved, remove them from the remaining $_POST fields
		*/
		unset( $_POST['fetp-themes'] );
		
		/* Remove paypal and other unnecessary fields */
		foreach( $_POST as $key => $value ) {
			if ( strstr( $key, 'paypal' ) || // paypal options
				strstr( $key, '_price' ) || // theme prices
				strstr( $key, 'nonce' ) || // plugin defined nonces
				strstr( $key, '_wp_http_referer' ) || // $_SERVER['HTTP_REFERER'];
				strstr( $key, 'fetp_htaccess' ) || // htaccess
				strstr( $key, 'fetp_update_options' ) || // submit button
				strstr( $key, 'action' ) // form action
			)
			unset( $_POST[ $key ] );
		}
		
		/* process the rest */
		$extra_fields = $_POST;
		
		$fetp_options = array();
		
		/*
		* save options
		* 
		* If a custom metabox has been removed, so will be it's data
		* This saves only the plugin default options and any other input
		* field value at time of form submission
		*/
		
		foreach( $extra_fields as $field => $value ) {
			$fetp_options[ $field ] = stripslashes( $value );
		}
		
		$fetp_options['themes'] = $fetp_themes;
		$fetp_options['premium'] = $premium_themes;
		$fetp_options['paypal'] = $paypal_settings;
		$fetp_options['last_cron'] = fetp_get_option('last_cron');
		
		update_option( 'fetp_options', $fetp_options );
		wp_redirect( 'admin.php?page=' . $this->slug . '&updated=1' );
		exit;
	}
	
	/* Display a success message on update */
	function options_updated() {
		echo ( isset( $_GET['updated'] ) ) ? '<div class="updated fade"><p>Options sucessfully <b>updated</b></p></div>' : '';
	}
	
	/* Add Default Metaboxes */
	function meta_boxes() {
	global $pagenow;
		
		$t = fetp_plugin_info();
		
		$metaboxes = array(
			1 => array(
				'id' => 'fetp_primary',
				'title' => 'Front End Theme Preview Settings',
				'callback' => array( $this, 'settings_meta_box' ),
				'post_type' => $this->post_type,
				'context' => 'normal',
				'priority' => 'high'
			),
			2 => array( 
				'id' => 'fetp_submit', 
				'title' => 'Update Options', 
				'callback' => array( $this, 'submit_form_meta_box' ), 
				'post_type' => $this->post_type, 
				'context' => 'side', 
				'priority' => 'default' 
			),
			3 => array(
				'id' => 'fetp_no_beer',
				'title' => $t['Name'],
				'callback' => array( $this, 'info' ),
				'post_type' => $this->post_type,
				'context' => 'side',
				'priority' => 'low'
			)
		);
		
		foreach( $metaboxes as $box => $args ) {
			$this->add_meta_box( $args );
		}
	}
	
	/* Options meta box content */
	public function settings_meta_box() {
	global $fetp_admin, $fetpay, $fetp;
		
		/* Get all Themes */
		$wp_themes = wp_get_themes();
		
		/* Preview list */
		$preview_list = fetp_get_option( 'themes' );
		
	?>
		<div id="fetp-admin-wrap">
			<div id="fetp-nav">
				<ul class="fetp-nav">
					<li><a href="#" class="general active">General</a></li>
					<li><a href="#" class="paypal">Paypal Settings</a></li>
				</ul>
			</div>
			
			<div class="fetp-clear"></div>
			
			<div id="fetp-tabs">
				<div class="fetp-tab fetp-general">				
					<div class="option">
					<input type="checkbox" <?php if ( fetp_get_option( 'cookie' ) ) echo 'checked'; ?> name="cookie" id="fetp_cookie" value="1" />
					<label for="fetp_cookie"> : Enable Cookies For Site Wide Theme Preview</label>
					</div>
					
					<div class="option">
					<input type="checkbox" <?php if ( fetp_get_option( 'select' ) ) echo 'checked'; ?> name="select" id="fetp_select" value="1" />
					<label for="fetp_select"> : Add Themes Dropdown To Preview Panel</label>
					</div>
					
					<div class="option">
					<input type="checkbox" <?php if ( fetp_get_option( 'enable_download' ) ) echo 'checked'; ?> value="1" name="enable_download"  id="fetp_enable_download" />
					<label for="fetp_enable_download"> : Enable Theme Download</label>
					</div>
					
					<div class="option">
					<input type="checkbox" <?php if ( fetp_get_option( 'delete_data' ) ) echo 'checked'; ?> value="1" name="delete_data" id="fetp_delete_data" /> :
					<label for="fetp_delete_data">Delete Plugin Data When You Deactivate</label>
					</div>
					
					<div class="option">
						<select name="clean_up_frequency" id="fetp_clean_up_frequency">
							<option value="hourly" <?php if ( fetp_get_option( 'clean_up_frequency' ) === 'hourly' ) echo 'selected'; ?>>Hourly</option>
							<option value="daily" <?php if ( fetp_get_option( 'clean_up_frequency' ) === 'daily' ) echo 'selected'; ?>>Daily</option>
							<option value="twicedaily" <?php if ( fetp_get_option( 'clean_up_frequency' ) === 'twicedaily' ) echo 'selected'; ?>>Twice Daily</option>
							<option value="off" <?php if ( fetp_get_option( 'clean_up_frequency' ) === 'off' ) echo 'selected'; ?>>Off</option>
						</select>
						<label for="fetp_clean_up_frequency">
							Select how often to clear your <code>wp-content/themes</code> folder of zipped files.
						</label>
					</div>
					
					<?php if ( $fetp->have_files() ) : ?>
					<div class="option cleanup">
						<a href="<?php echo fetp_seturl( array( 'fetp-ajax-clean-up' => '', 'fetp_clean_up' => '' ) ); ?>" class="button-primary fetp-clean-up">Clean Up Themes Folder</a>
						<span class="fade" style="margin-left: 10px; font: 13px sans-serif;"></span>
					</div>
					<?php endif; ?>
					
					<h3>Add Themes To Preview List</h3>
					<p class="description" style="padding-left: 15px;">For Premium Themes, set the price on the adjacent price field</p>
					
					<div id="options-themes-list" class="option last">
						<table class="wp-list-table widefat fixed fetp posts" id="fetpall">
							<thead>
								<th style="width: 5%!important;">#</th>
								<th style="width: 3%!important; text-align: center!important;"><input title="select all" type="checkbox" class="fetp-all" id="all" onclick="fetpCheckAll(this.id, 'fetpall', 'checkall')" /></th>
								<th>Theme</th>
								<th>Price</th>
							</thead>
							
							<?php 
								$i = 0;
								$alternate = 0;
								foreach( $wp_themes as $theme ) : 
								$i++; 
								$class = ( $alternate++ % 2 == 0 ) ? 'alternate' : '';
							?>
								<tr class="<?php echo $class; ?> themes-list">
									<th><?php echo $i .'.'; ?></th>
									<th style="width: 3%!important; text-align: center!important;">
										<input class="checkall" type="checkbox" <?php if ( ! empty( $preview_list ) && in_array( $theme->stylesheet, $preview_list ) ) echo 'checked'; ?> name="fetp-themes[]" id="<?php echo $theme->stylesheet; ?>" value="<?php echo $theme->stylesheet; ?>" />
									</th>
									<th>									
										<label for="<?php echo $theme->stylesheet; ?>"><?php echo $theme->Name; ?></label>
									</th>
									<td>
										<label for="<?php echo $theme->stylesheet . '_price'; ?>"><span class="currency-symbol"><?php echo $fetpay->cursymbol; ?></span>
										<input size="8" class="theme-price" type="text" value="<?php echo $fetpay->price( $theme->stylesheet ); ?>" name="<?php echo $theme->stylesheet . '_price'; ?>" id="<?php echo $theme->stylesheet . '_price'; ?>" />
										</label>
									</td>
								</tr>
							<?php endforeach; ?>
								<div id="default-theme"></div>
						</table>
					</div>
					
				</div>
				
				<div class="fetp-tab fetp-paypal hidden">
					
					<div class="option last">
						<table class="wp-list-table widefat fixed fetp plugins">
							<thead>
								<th>Option</th>
								<th>Value</th>
							</thead>
							
							<tr class="active">						
								<th>Your Paypal Email Address</th>
								<td><input type="text" size="25" name="paypal_email" value="<?php echo fetp_paypal_setting( 'business' ); ?>" /></td>
							</tr>
							
							<tr class="inactive">
								<th>Enable Sandbox Testing</th>
								<td><input type="checkbox" value="1" <?php if ( fetp_paypal_setting( 'sandbox' ) ) echo 'checked'; ?> name="paypal_sandbox" />
								Check this option if you want to use the PayPal sandbox testing. You will need to create an account from the 
								<a href="<?php echo $fetpay->sandbox_uri(); ?>">PayPal Developer Site</a>.
								<br />
								<br />
								Remember to uncheck this when you're done testing.
								</td>
							</tr>
							
							<tr class="active">
								<th>Currency</th>
								<td>
									<input type="text" size="2" name="paypal_currency" value="<?php echo fetp_paypal_setting( 'currency' ); ?>" /> 
									<small>(e.g USD, EUR, GBP, AUD)</small>
								</td>
							</tr>
							
							<tr class="inactive">
								<th>Currency Symbol</th>
								<td>
									<input type="text" size="1" value="<?php echo fetp_paypal_setting( 'cursymbol' ); ?>" name="paypal_currencysymbol"/> 
									<small>(e.g. $, &#163;, &#8364;) </small>
								</td>
							</tr>
							
							<tr class="active">
								<th>Your Country Code</th>
								<td>
									<input type="text" size="2" value="<?php echo fetp_paypal_setting( 'location' ); ?>" name="paypal_location"/> 
									<small>(e.g. US, KE) </small>
								</td>
							</tr>
							
							<tr class="inactive">
								<th>Transaction Log</th>
								<td>
									<input type="checkbox" value="1" <?php if ( fetp_paypal_setting( 'ipn_log' ) ) echo 'checked'; ?> name="paypal_log" /> 
									Check this to log IPN errors for debug.
								</td>
							</tr>
							
							<tr class="active">
								<th>Email Options</th>
								<td>
									<strong>Subject</strong>
									<br />
									<input type="text" name="paypal_subject" value="<?php echo fetp_paypal_setting( 'subject' ); ?>" size="35" />
									<br />
									<br />
									<strong>Message</strong>
									<br />
									<textarea name="paypal_message" rows="6"><?php echo fetp_paypal_setting( 'message' ); ?></textarea>
									<br />
									After a successful purchase, we'll register a user (if their email is not already registered) in your site to enable them access to their themes. 
									You can specify the email subject/message here and the plugin will take care of the rest. i.e registering the user and sending them 
									the login details.
								</td>
							</tr>
							
							<tr class="inactive">
								<th>HTACCESS FILE</th>
								<td>
									<strong>DON'T edit this unless you know what you're doing. This prevents direct access to your themes</strong>
									<br />
									<br />
									<textarea name="fetp_htaccess"><?php fetp_htaccess(); ?></textarea>
								</td>
							</tr>
						</table>
					</div>
				</div>
			</div>
			
			<div class="fetp-clear"></div>
		</div>
	<?php
	}

	/*
	* submit_button() metabox
	*/
	function submit_form_meta_box() {
		$content = '<p class="submit">
			<input type="submit" name="fetp_update_options" value="Update Options" class="button-primary fetp-save" />
			<span class="saving hidden">Saving Options...</span>
		</p>';
		
		/* Serve the filtered content with red wine */
		echo apply_filters( 'fetp_submit_form_metabox', $content ) . '<div class="fetp-clear"></div>';
	}
	
	/*
	* Plugin info
	*/
	function info() {
		
		$plugin = fetp_plugin_info();
		
		$link = fetp_support_project(true);
		
		$no_beer = array(
			1 => __( 'The developer doesn\'t actually drink beer and would rather you buy him a raccoon! Don\'t ask!', 'fetp' ),
			2 => __( 'The developer isn\'t an alcoholic, but appreciates the gesture. Next time please buy him a steak: medium rare, or medium, or rare... whatever, just cook it!', 'fetp' )
		);
		
		echo '<table>
			<thead>
				<th></th>
				<th></th>
			</thead>
			
			<tr>
				<th>Version:</th>
				<td>' . $plugin['Version'] . '</td>
			</tr>
			
			<tr>
				<th>Author:</th>
				<td><a href="'. $plugin['AuthorURI'] .'">' . $plugin['Author'] .'</td>
			</tr>
		</table>
		
		<p>
			If you like this plugin, you can buy the developer a beer or two <a href="' . $link . '">here</a>.
			<br />
			<br />
			<span class="description no-beer">' . $no_beer[ mt_rand( 1, count( $no_beer ) ) ] . '</span>
		</p>
		
		<p><strong>!!</strong> You can hide me at any time via the screen options up there.</p>
		';
	}
}

/** Extender Class Example **/

/*
* new MY_Extender(); 
*/

/**** ^ REMOVE THE COMMENT ABOVE TO LOAD THE EXAMPLE CLASS ****/

class MY_Extender extends Fetp_Admin {
	
	/* Class constructor */
	public function __construct() {
		
		/* register meta box(es) */
		add_action( 'add_meta_boxes', array( $this, 'my_meta_box' ) );
		
		/* Add custom content to the side submit form metabox */
		add_filter( 'fetp_submit_form_metabox', array( $this, 'my_submit_box_filter' ) );
	}
	
	/* Define your meta box */
	function my_meta_box() {
		
		/* 
		* This array parameter expects a minimum of six keys and should
		* be defined as, 'id', 'title', 'callback', 'post_type', 'context', 'priority' 
		* in no particular order for the "parent::add_meta_box" to 'extract()'
		*/
		$metaboxes = array( 
			1 => array(
				'id' => 'my_metabox_id',
				'title' => 'My Metabox Title',
				'callback' => array( $this, 'box_one' ),
				'post_type' => $this->post_type, // this plugin's page
				'context' => 'side',
				'priority' => 'high'
			),
			2 => array(
				'title' => 'My Normal Metabox',
				'priority' => 'low',
				'post_type' => $this->post_type,
				'id' => 'my_second_box',
				'callback' => array( $this, 'box_two' ),
				'context' => 'normal'
			)
		);
		
		foreach( $metaboxes as $key => $args ) {
			$this->add_meta_box( $args );
		}
	}
	
	/* Metabox 1 content */
	function box_one() {
		echo '<p>My Metabox Content</p>';
		echo '<label>My Email Field <input type="email" name="my_email" value="'.fetp_get_option('my_email').'" /></label>';
		echo '<p>To get the value for the above input field, use <code>fetp_get_option( \'my_email\' );</code></p>';
	}
	
	/* Metabox 2 content */
	function box_two() {
		echo '<p>My Metabox Content</p>';
		echo '<label>My Input Field <input type="text" name="box_two" value="'.fetp_get_option('box_two').'" /></label>';
		echo '<p>To get the value for the above input field, use <code>fetp_get_option( \'box_two\' );</code></p>';		
	}
	
	function my_submit_box_filter( $content ) {
		return 'Content added via the <code>fetp_submit_form_metabox</code> filter' . $content;
	}
}

/* 
* Get plugin option 
*
* @param string - plugin settings option
* @return array|int|string option value
*/
function fetp_get_option( $option ) {
	
	$fetp_options = get_option( 'fetp_options' );
	
	return ( isset( $fetp_options[ $option ] ) ) ? $fetp_options[ $option ] : false;
}

/* 
* Delete plugin option 
*
* @param $option (string) - option name
*/
function fetp_delete_option( $option ) {
	$fetp_options = get_option( 'fetp_options' );
	
	if ( isset( $fetp_options[ $option ] ) )
		unset( $fetp_options[ $option ] );
	
	update_option( 'fetp_options', $fetp_options );
}

/*
* Get paypal setting
*
* @param $setting - string. The paypal setting name
* @return string $setting content | (bool)false
*/
function fetp_paypal_setting( $setting ) {
global $fetpay;	
	/* get all paypal options */
	$paypal = fetp_get_option( 'paypal' );
	
	if ( isset( $paypal[ $setting ] ) && ! empty( $paypal[ $setting ] ) ) {
		$setting = $paypal[ $setting ];
	} elseif ( isset( $fetpay->$setting ) ) {
		$setting = $fetpay->$setting;
	} else {
		$setting = false;
	}
	
	return $setting;
}

/*
* Update previous options to a single array to reduce options clatter
* @return void
*/
function fetp_set_new_options() {
	$themes_prices = array();
	
	$themes = get_option('fetp_themes');
	
	foreach( $themes as $theme ) {
		$themes_prices[ $theme ] = get_option( $theme . '_price' );
	}
	
	$options = array(
		'cookie' => get_option( 'fetp_cookie' ),
		'themes' => get_option( 'fetp_themes' ),
		'select' => get_option( 'fetp_select' ),
		'last_cron' => get_option( 'fetp_last_cron' ),
		'clean_up_frequency' => get_option( 'fetp_clean_up_frequency' ),
		'enable_download' => get_option( 'fetp_enable_download' ),
		'paypal_settings' => get_option( 'fetp_paypal_settings' ),
		'premium' => $themes_prices,
		'delete_data' => get_option( 'fetp_delete_data' )
	);

	update_option( 'fetp_options', $options );
	
	/* Delete Old Options */
	$wp_themes = wp_get_themes();
	
	delete_option( 'fetp_cookie' );
	delete_option( 'fetp_themes' );
	delete_option( 'fetp_select' );
	delete_option( 'fetp_last_cron' );
	delete_option( 'fetp_clean_up_frequency' );
	delete_option( 'fetp_enable_download' );
	delete_option( 'fetp_enable_default_theme_download' );
	delete_option( 'fetp_paypal_settings' );
	delete_option( 'fetp_premium' );
	delete_option( 'fetp_delete_data' );		
	foreach( $wp_themes as $theme ) {
		delete_option( $theme->stylesheet . '_price' );
	}
}

/* Get the htaccess file */
function fetp_htaccess() {
$htaccess = "<Files ~ \".zip$\">
	Order allow,deny
	Deny from all
</Files>
";
	$file = WP_CONTENT_DIR . '/themes/.htaccess';
	echo ( file_exists( $file ) ) ? stripslashes( file_get_contents( $file ) ) : stripslashes( $htaccess );
}

/* Write the htaccess file */
function fetp_write_htaccess() {
$htaccess = '<Files ~ ".zip$">
	Order allow,deny
	Deny from all
</Files>
';
	$file = WP_CONTENT_DIR . '/themes/.htaccess';
	
	if ( ! file_exists( $file ) ) {
		fwrite( fopen( $file, 'w+' ), $htaccess );
	}
}

/* PayPal Donations */
function fetp_support_project( $link = false ) {
	$form = '<form action="https://www.paypal.com/cgi-bin/webscr" method="post" target="_blank">
<input type="hidden" name="cmd" value="_s-xclick">
<input type="hidden" name="hosted_button_id" value="H7MVYPXN8BUSL">
<input type="image" src="https://www.paypalobjects.com/en_US/i/btn/btn_donate_LG.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online!">
<img alt="" border="0" src="https://www.paypalobjects.com/en_US/i/scr/pixel.gif" width="1" height="1">
</form>';
	
	$_link = 'https://www.paypal.com/cgi-bin/webscr/?cmd=_s-xclick&hosted_button_id=H7MVYPXN8BUSL';
	
	if ( $link ) {
		return $_link;
	} else {
		return $form;
	}
}

// write our access file
fetp_write_htaccess();
ob_flush();