<?php

ob_start();

/* 
* This Class contains the required methods to enable on the fly
* theme zipping and download and file delete at specified intervals
*/

$fetp = new FrontendThemePreview;

class FrontendThemePreview {
	
	/* Regex */
	var $regex = '|[^a-z0-9_./-]|i';
	
	function __construct() {
		
		// Register Shortcodes
		self::shortcodes();
		
		// Listen for theme downloads
		self::theme_download();
		
		self::premium_download();
		
		// 'Cron' 
		self::do_clean_up_cron();
		
		// Clean up
		self::do_clean_up();
		
		// Listen for switch theme requests
		self::switch_theme();
	}
	
	/* Zip The Desired Theme	*/	
	public function zip_theme( $stylesheet ) {
		/* Get Theme Details */
		$theme = wp_get_theme( $stylesheet );
		
		/* Get listed themes */
		$fetp_themes = fetp_get_option( 'themes' );
		
		/* No need to zip an existing archive */
		if ( ! file_exists( WP_CONTENT_DIR . "/themes/" . $stylesheet . "." . $theme->Version . ".zip" ) ) {
			
			// let's roll
			if ( $theme->exists() &&  in_array( $stylesheet, $fetp_themes ) ) : // our theme must actually exist and allowed for download
				
				/* 
				* extend max_execution_time: we don't
				* want our zip process to run out of juice 
				*/
				ini_set( "max_execution_time", 300 );
				
				// define theme to zip and location
				$the_folder = WP_CONTENT_DIR . '/themes/'.$stylesheet;
				$zip_file_name = WP_CONTENT_DIR . '/themes/'.$stylesheet.'.'.$theme->Version.'.zip';

				$zip = new FetpZipArchive;

				$res = $zip->open( $zip_file_name, ZipArchive::CREATE );

				if ( $res === TRUE ) {
				
					$zip->addDir( $the_folder, basename( $the_folder ) );
					
					$zip->close();
				}
			endif;			
		}
	}
	
	/* Download Headers */
	public function download_headers( $file_name, $path ) {
		header( "Pragma: public" );
		header( "Expires: 0" );
		header( "Cache-Control: must-revalidate, post-check=0, pre-check=0" );
		header( "Cache-Control: public" );
		header( "Content-type: application/octet-stream" );
		header( "Content-Disposition: attachment; filename=\"".$file_name."\"" );
		header( "Content-Transfer-Encoding: binary" );
		header( "Content-Length: " . filesize( $path . $file_name ) );
	}
	
	/* Theme  Download */
	public function theme_download() {
		
		// theme to download
		$stylesheet = fetp_get_theme();
		
		// No need to to run this if themes are not up for download
		if ( fetp_get_option( 'enable_download' ) && ! $this->is_premium( $stylesheet ) ) {
			
			// get theme info
			$theme = wp_get_theme( $stylesheet );
			
			// is it in download list
			$fetp_themes = fetp_get_option( 'themes' );
			
			if ( isset( $_GET['fet_download'] ) && in_array( $stylesheet, $fetp_themes ) ) :
				
				// find zip file
				if ( file_exists( WP_CONTENT_DIR . '/themes/'.$stylesheet.'.'.$theme->Version.'.zip' ) ) {
					
					$file_name = $stylesheet . '.' . $theme->Version . ".zip";
					$path = WP_CONTENT_DIR . "/themes/";
					
					$_SESSION['fetp_downloading'] = true;
					
					//header( 'Location: ' . content_url() . '/themes/'.$stylesheet.'.'.$theme->Version.'.zip' );
					$this->download_headers( $file_name, $path );
					ob_end_flush();
					readfile( $path . $file_name );
					unset( $_SESSION['fetp_downloading'] );
					
				} else {
					
					// A theme may have been deleted :(
					if ( ! wp_get_theme( $stylesheet )->exists() )
						return false;
					
					$this->zip_theme( $stylesheet );
					$file_name = $stylesheet . '.' . $theme->Version . ".zip";
					$path = WP_CONTENT_DIR . "/themes/";
					
					$_SESSION['fetp_downloading'] = true;
					//header( 'Location: ' . content_url() . '/themes/'.$stylesheet.'.'.$theme->Version.'.zip' );
					$this->download_headers( $file_name, $path );
					ob_end_flush();
					readfile( $path . $file_name );
					unset( $_SESSION['fetp_downloading'] );
				}
			endif;
		}
	}
	
	/* premium theme download */
	public function premium_download() {
	global $wpdb, $current_user;
		require_once( ABSPATH . '/wp-includes/pluggable.php' );
		get_currentuserinfo();
		
		
		$path = WP_CONTENT_DIR . '/themes/';
		
		if ( is_user_logged_in() ) :
			
			$email = $current_user->user_email;
			
			if ( isset( $_GET['get_theme'] ) && !empty( $_GET['get_theme'] ) ) {
				
				$stylesheet = preg_replace( $this->regex, '', $_GET['get_theme'] );
				$data = wp_get_theme( $stylesheet );
				$file = $stylesheet . '.' . $data->Version . '.zip';
				
				// get themes for current user
				$stylesheets = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . FETP_TABLE . " WHERE user_email = %s", $email ), ARRAY_A );
				$stylesheets = $stylesheets['user_themes'];
				
				if ( strpos( $stylesheets, ',' ) ) {
					
					$themes = explode( ',', $stylesheets );
					
				} elseif ( !strpos( $stylesheets, ',' ) && $stylesheets != NULL ) {
					
					$themes = array( $stylesheets );
				} else {
					
					// if the current user hasn't paid for this theme, return an empty array.
					$themes = array();
				}
				
				// finally serve the theme
				if ( in_array( $stylesheet, $themes ) ) {
					
					// first search for existing files
					if ( file_exists( $path . '/' . $stylesheet . '.' . $data->Version . '.zip' ) ) {
						
						$_SESSION['fetp_downloading'] = true;
						
						$this->download_headers( $file, $path );
						ob_end_flush();
						readfile( $path . $file );
						unset( $_SESSION['fetp_downloading'] );
						
					// our file is missing, let's create it
					} else {
						
						// A theme may have been deleted :(
						if ( ! wp_get_theme( $stylesheet )->exists() )
							return false;
						
						$this->zip_theme( $stylesheet );
						$_SESSION['fetp_downloading'] = true;
						
						$this->download_headers( $file, $path );
						ob_end_flush();
						readfile( $path . $file );
						unset( $_SESSION['fetp_downloading'] );
					}
				}
			}
		endif; // user logged in
	}
	
	/* Add Shortcodes	*/
	
	/*
	* @shortcode [fetpreview theme="$stylesheet"]
	*/
	public function preview( $atts ) {
		return '<a class="fetp-preview" href="'. fetp_seturl( 'fet_preview', $atts['theme'] ) .'">Preview</a>';
	}
	
	/*
	* @shortcode [fetpdownload theme="$stylesheet"]
	*/
	public function download( $atts ) {
	global $fetpay;
		
		$stylesheet = $atts['theme'];
		
		if ( $this->is_premium( $stylesheet ) ) {
			return '<a class="add-theme-to-cart" href="'.fetp_buy_url( $stylesheet, true ).'">(' . $fetpay->cursymbol. $fetpay->price( $stylesheet ) . ') Add To Cart</a>';
		} else {
			return '<a class="fetp-download" href="' . fetp_seturl( 'fet_download', $stylesheet ) . '">Download</a>';
		}		
	}
	
	/*
	* Alternatively use [fetp] to display a preview and a 
	* download link together wrapped in a paragraph or div
	* @shortcode [fetp theme="$stylesheet" wrapper="p"]
	*/
	public function fetp( $atts ) {
	global $fetpay;
		$wrap = ( isset( $atts['wrapper'] ) ) ? $atts['wrapper'] : 'p';
		$stylesheet = $atts['theme'];
		
		if ( $this->is_premium( $stylesheet ) ) {
			$DText = '<a class="add-theme-to-cart" href="'.fetp_buy_url( $stylesheet, true ).'">(' . $fetpay->cursymbol. $fetpay->price( $stylesheet ) . ') Add To Cart</a>';
		} else {
			$DText = '<a href="?fet_download='.$stylesheet.'">Download</a>';
		}
		
		if ( $wrap == 'p' ) :
			return '<p align="center"><a href="'. fetp_seturl( 'fet_preview', $stylesheet ) .'">Preview</a> | '. $DText .'</p>';
		elseif ( $wrap == 'div' ) :
			return '<div class="fetp-prev-dnl"><a href="'. fetp_seturl( 'fet_preview', $stylesheet ) .'">Preview</a> | '. $DText .'</div>';
		else:
			return '<p align="center"><a href="'. fetp_seturl( 'fet_preview', $stylesheet ) .'">Preview</a> | '. $DText .'</p>';
		endif;
	}
	
	/*
	* Display A drop down of available in a sidebar widget 
	*
	* @shortcode [fetpselect] 
	*/
	public function select_preview() {
		
		$fetp_themes = fetp_get_option( 'themes' );
		
		/* Remove deleted themes */
		foreach( $fetp_themes as $key => $template ) {
			if ( ! wp_get_theme( $template )->exists() ) {
				unset( $fetp_themes[ $key ] );
			}
		}
		
		$select = '<select class="fetp-select" name="fet_preview" onchange="window.location.href=this.value;">';
		$select .= '<option value="">-- Select Theme --</option>';
			foreach( $fetp_themes as $theme ) {
				$select .= '<option '.fetp_selected($theme).' value="' . fetp_seturl( 'fet_preview', $theme ) . '">'.wp_get_theme( $theme )->Name.'</option>';
			}
		$select .= '</select>';
		
		return $select;
	}
	
	/* Premium themes shortcode */
	function list_premium_themes() {
		
		/* End road of the road if user isn't logged in */
		if ( ! is_user_logged_in() )
			return '<p>You need to be <a href="' . wp_login_url( $_SERVER['REQUEST_URI'] ) . '">logged in</a> to view this resource</p>';
		
		global $fetpay, $current_user, $wpdb;
		get_currentuserinfo();
		
		$email = $current_user->user_email;
		
		$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . FETP_TABLE . " WHERE user_email = %s", $email ), ARRAY_A );
		
		if ( strpos( $row['user_themes'], ',' ) ) {
			$themes = explode( ',', $row['user_themes'] );
			
			// A theme may have been deleted
			
			/*
			foreach( $themes as $key => $theme ) {
				if ( ! wp_get_theme( $theme )->exists() )
					unset( $themes[ $key ] );
			}
			*/
			
			$list = '<ul>';
			foreach( $themes as $stylesheet ) {
				
				// check for deleted themes
				$deleted_theme = apply_filters( 'fetp_deleted_theme', '<li>The Requested theme wasn\'t found in our database. Sorry</li>' );
				
				$list .= ( wp_get_theme( $stylesheet )->exists() ) ? '<li>' . wp_get_theme( $stylesheet )->Name . ' &#8212; <a href="?get_theme='.$stylesheet.'">Download</a></li>' : $deleted_theme;
			}
			$list .= '</ul>';
			
		} elseif ( ! strpos( $row['user_themes'], ',' ) && $row['user_themes'] != NULL ) {
				
				// Check for deleted themes
				if ( wp_get_theme( $row['user_themes'] )->exists() ) {
					$list = '<ul><li>' . wp_get_theme( $row['user_themes'] )->Name . ' &#8212; <a href="?get_theme='.$row['user_themes'].'">Download</a></li></ul>';
				} else {
					$list .= apply_filters( 'fetp_deleted_theme', '<ul><li>The Requested theme wasn\'t found in our database. Sorry</li></ul>' );
				}
			
		} else {
			
			$list .= '<ul><li>Nothing found in your theme database</li></ul>';
		}
		
		/* Serve with fried chicken */
		return $list;
	}
	
	/* Register Shortcodes */
	function shortcodes() {
		add_shortcode( 'fetpreview', array( &$this, 'preview' ) );
		add_shortcode( 'fetpdownload', array( &$this, 'download' ) );
		add_shortcode( 'fetp', array( &$this, 'fetp' ) );
		add_shortcode( 'fetpselect', array( &$this, 'select_preview' ) );
		add_shortcode( 'fetplistpremium', array( &$this, 'list_premium_themes' ) );
		add_filter( 'widget_text', 'do_shortcode' );
	}
	
	/* Check if we have zip files created by this plugin */
	public function have_files() {
		$themes = wp_get_themes();
		$path = WP_CONTENT_DIR . '/themes';
		
		foreach( $themes as $theme ) {
			if ( file_exists( $path . '/' . $theme->stylesheet . '.' . $theme->Version . '.zip' ) ) {
				return true;
			}
		}
		
		return false;
	}
	
	/* Number of zip files */
	public function num_files( $count = false ) {
		$themes = wp_get_themes();
		$path = WP_CONTENT_DIR . '/themes';
		
		$num = 0;
		foreach( $themes as $theme ) {
			if ( file_exists( $path . '/' . $theme->stylesheet . '.' . $theme->Version . '.zip' ) ) {
				$num++;
			}
		}
		
		if ( $num > 1 ) {
			return "<strong>$num</strong> Zip Files in <code>" . content_url('/themes/') . "</code>";
		} elseif ( $num == 1 ) {
			return "<strong>$num</strong> Zip File Found in <code>" . content_url('/themes/') . "</code>";
		} elseif ( $count ) {
			return $num;
		}
	}
	
	/* Folder Clean Up */
	private function clean_up() {
		$dir = WP_CONTENT_DIR . '/themes';
		
		// apply permission to themes folder
		//chmod( $dir, 0777 );
		
		$themes = wp_get_themes();
		
		foreach( $themes as $theme ) {
			if ( is_file( $dir . '/'. $theme->stylesheet .'.'. $theme->Version .'.zip' ) && ! $this->is_download() ) {
				// apply permission to zip file..
				chmod( $dir . '/'. $theme->stylesheet .'.'. $theme->Version .'.zip', 0777 );
				// ..then delete it
				unlink( $dir . '/'. $theme->stylesheet .'.'. $theme->Version .'.zip' );
			}
		}
		
		/* let's clear the IPN Log while we're at it */
		$fp = fopen( FETP_IPN_LOG_FILE, 'w+' );
		fwrite( $fp, '' );
		fclose( $fp );
	}
	
	public function do_clean_up() {		
		// ajax cleanup
		if ( isset( $_GET['fetp-ajax-clean-up'] ) && is_admin() ) :
			if ( $this->have_files() ) {
				$this->clean_up();
				echo 'OK';
				exit;
			} elseif ( ! $this->have_files() ) {
				echo '0';
				exit;
			} else {	
				echo 'error';
				exit;
			}
		endif;
		
		// manual cleanup
		if ( isset( $_GET['fetp_clean_up'] ) )
			$this->clean_up();
	}
	
	/*
	* Setup a 'cron job' to delete zip files
	*
	* Since 1.1
	*
	* Does not use wp's cron. God knows I tried to use it
	* and he forgave me. So should you :|
	*/
	public function do_clean_up_cron() {
		$recurrence = fetp_get_option( 'clean_up_frequency' );
		$last_cron = fetp_get_option( 'last_cron' );
		$now = time();
		
		$plugin_options = get_option( 'fetp_options' );
		
		switch( $recurrence ) :
			case 'hourly':
				if ( $last_cron < $now - 3600 ) {
					$this->clean_up();
					$plugin_options['last_cron'] = $now;
				}
			break;
			
			case 'daily':
				if ( $last_cron < $now - 86400 ) {
					$this->clean_up();
					$plugin_options['last_cron'] = $now;
				}
			break;
			
			case 'twicedaily':
				if ( $last_cron < $now - 43200 ) {
					$this->clean_up();
					$plugin_options['last_cron'] = $now;
				}
			break;
		endswitch;
		
		update_option( 'fetp_options', $plugin_options );
	}
	
	/* Get Next Cron */
	public function get_next_cron() {
		$last_cron = fetp_get_option( 'last_cron' );
		$recurrence = fetp_get_option( 'clean_up_frequency' );
		
		switch( $recurrence ) :
			case 'hourly':
				$next_cron = $last_cron + 3600 - time();
			break;
			
			case 'twicedaily':
				$next_cron = $last_cron + 43200 - time();
			break;
			
			case 'daily':
				$next_cron = $last_cron + 86400 - time();
			break;
			
			case 'off':
				$next_cron = 'off';
			break;
		endswitch;
		
		return $next_cron;
	}

	/* TimeLeft */
	public function get_cron_timeleft() {
		$cron = $this->get_next_cron();
		if ( 'off' == $cron ) {
			return null;
		} else {
			return $cron;
		}
	}
	
	/* 
	* Halt clean up if a download is in progress. The effectiveness
	* of this is not guaranteed but it's not life threatening... so far.
	*/
	public function is_download() {		
		if ( isset( $_SESSION['fetp_downloading'] ) ) {
			return true;
		} else {
			return false;
		}
	}
	
	/* Options to switch themes for admins */
	public function switch_theme() {
	require_once( ABSPATH . '/wp-includes/pluggable.php' );
		
		if ( isset( $_GET['fet_switcher'] ) && current_user_can( 'switch_themes' ) ) {
			
			$stylesheet = preg_replace( $this->regex, '', $_GET['fet_switcher'] );
			
			/* 
			* Make sure theme exists. It should work but it doesn't hurt to be thorough
			*/			
			if ( ! empty( $stylesheet ) && wp_get_theme( $stylesheet )->exists() ) {
				switch_theme( $stylesheet );
				fetp_delete_cookie();
				wp_redirect( remove_query_arg( fetp_requests() ) );
				exit;
			}
		}
	}

	/* Check if is a Premium theme */
	public function is_premium( $stylesheet ) {
		
		/* Premium themes */
		$premium = fetp_get_option( 'premium' );
		
		if ( empty( $premium ) )
			return false;
		
		if ( isset( $premium[ $stylesheet ] ) && ! empty( $premium[ $stylesheet ] ) )
			return true;
		
		return false;
	}
}
ob_flush();