<?php

ob_start();

/* FETP ADMIN */

// hook options page
add_action( 'admin_menu', 'fetp_admin_menu' );

function fetp_admin_menu() {	
	/* Hook the admin */
	$fetpage = add_menu_page( __( 'Fetp Options' ), __( 'Fetp Options' ), 'manage_options', 'fetp', 'fetp_options', plugins_url( '/front-end-theme-preview/img/admin-icon.png' ) );
	
	/* Add the help tab */
	add_action( "load-$fetpage", "fetp_help" );
}

/** Help tab contents **/
function fetp_help() {

	$screen = get_current_screen();
	
	$overview = '<h3><strong>Front End Theme Preview By <a href="http://profiles.wordpress.org/harvey_j">Harvey J</a></strong></h3>';
	$overview .= '<p>This plugin allows site visitors to preview your precious themes with the added options to download/buy the theme(s).</p>';
	$overview .= '<p>Tweak the plugin with the options below.</p>';
	
	$screen->add_help_tab(
		array(
			'id' => 'fetp',
			'title' => __( 'Overview', 'fetp' ),
			'content' => $overview
		)
	);
	
	$usage = '<h3><strong>Usage</strong></h3>';
	$usage .= '<p>To get things going, make sure to add the themes you want to make available on the checkbox list below</p>';
	$usage .= '<p>To initiate a preview, load your site like: <code>' . site_url( '/?fet_preview=twentytwelve' ) . '</code></p>';
	$usage .= '<p>Site Admins have a bonus option of changing the theme from the front end. See the preview toolbar.</p>';
	$usage .= '<p>There are three widgets which you can place in your sidebar to list the available themes in a dropdown select, another to show the user\'s purchased themes and lastly another widget that displays the plugin\'s paypal shopping cart';
	
	$screen->add_help_tab(
		array(
			'id' => 'fetp_usage',
			'title' => __( 'Usage', 'fetp' ),
			'content' => $usage
		)
	);
	
	$codes = '<h3><strong>Shortcodes</strong></h3>';
	$codes .= '<ul>';
	$codes .= '<li>[fetp] &#8212; Displays download/buy and preview links</li>';
	$codes .= '<li>[fetpreview theme="$stylesheet"] &#8212; Displays a preview link</li>';
	$codes .= '<li>[fetpdownload theme="$stylesheet"] &#8212; Displays a download or a buy link</li>';
	$codes .= '<li>[fetpselect] &#8212; Displays a dropdown of available themes which you can place in a sidebar</li>';
	$codes .= '<li>[fetplistpremium] &#8212; Displays the logged in user\'s purchased themes</li>';
	$codes .= '</ul>';
	
	$screen->add_help_tab(
		array(
			'id' => 'fetp_codes',
			'title' => __( 'Shortcodes', 'fetp' ),
			'content' => $codes
		)
	);
	
	$hooks = '<h3><strong>Hooks</strong></h3>';
	$hooks .= '<p>To add a custom option to the toolbar like a link or plain text, use the <code>fetp_action</code> hook.</p>';
	$hooks .= '<p>If a premium theme has been deleted, use the <code>fetp_deleted_theme</code> hook to notify the user in the Fetp_Download_Themes Widget</p>';
	$hooks .= '<p>More hooks later...</p>';
	
	$screen->add_help_tab(
		array(
			'id' => 'fetp_hooks',
			'title' => __( 'Hooks', 'fetp' ),
			'content' => $hooks
		)
	);
	
	$premium_tab = '<h3><strong>Premium Theme Settings</strong></h3>';
	$premium_tab .= '<p>The premium themes tab below has two input fields: the checkbox and the price text input. To add a theme to the premium list, you must check against the theme then price it on the right. Hit "Update Options" and you\'re ready to make some money :)</p>';
	$premium_tab .= '<p><strong>NOTE: </strong> Theme prices should have a two decimal place amount.</p>';
	
	$screen->add_help_tab(
		array(
			'id' => 'fetp_premium',
			'title' => __( 'Premium Themes', 'fetp' ),
			'content' => $premium_tab
		)
	);
	
	$screen->set_help_sidebar(
		'<h3><strong>For Questions:</strong></h3>' . 
		'<p><a href="http://wordpress.org/plugins/front-end-theme-preview">Check the Support Forum</a></p>' .
		
		'<h3><strong>For a pat on the back:</strong></h3>' .
		'<p><a href="'. fetp_support_project( true ) .'">Pass by the donation box on your way home</a></p>'
	);
}

// write our access file early enough
fetp_write_htaccess();

function fetp_options() {
global $fetp, $fetpay;
	
	$fetp_options = array();

	// Save options
	if ( isset( $_POST['fetp_save_options'] ) ) {
		
		if ( !empty( $_POST['fetp_cookie'] ) ) {
			update_option( 'fetp_cookie', $_POST['fetp_cookie'] );
		} else {
			delete_option( 'fetp_cookie' );
		}
		
		if ( !empty( $_POST['fetp_select'] ) ) {
			update_option( 'fetp_select', $_POST['fetp_select'] );
		} else {
			delete_option( 'fetp_select' );
		}
		
		if ( !empty( $_POST['fetp_themes'] ) ) {
			update_option( 'fetp_themes', $_POST['fetp_themes'] );
		} else {
			update_option( 'fetp_themes', array() );
		}
		
		if ( !empty( $_POST['fetp_enable_download'] ) ) {
			update_option( 'fetp_enable_download', $_POST['fetp_enable_download'] );
		} else {
			delete_option( 'fetp_enable_download' );
		}
		
		if ( !empty( $_POST['fetp_enable_default_theme_download'] ) ) {
			update_option( 'fetp_enable_default_theme_download', $_POST['fetp_enable_default_theme_download'] );
		} else {
			delete_option( 'fetp_enable_default_theme_download' );
		}
		
		if ( ! empty( $_POST['fetp_delete_data'] ) ) {
			update_option( 'fetp_delete_data', $_POST['fetp_delete_data'] );
		} else {
			delete_option( 'fetp_delete_data' );
		}
		
		if ( $_POST['fetp_clean_up_frequency'] == 'hourly' && get_option( 'fetp_clean_up_frequency' ) != 'hourly' ) {
			update_option( 'fetp_clean_up_frequency', 'hourly' );
			update_option( 'fetp_last_cron', time() - 3600 );
		} elseif ( $_POST['fetp_clean_up_frequency'] == 'twicedaily' && get_option( 'fetp_clean_up_frequency' ) != 'twicedaily' )  {
			update_option( 'fetp_clean_up_frequency', 'twicedaily' );
			update_option( 'fetp_last_cron', time() - 43200 );
		} elseif ( $_POST['fetp_clean_up_frequency'] == 'daily' && get_option( 'fetp_clean_up_frequency' ) != 'daily' ) {
			update_option( 'fetp_clean_up_frequency', 'daily' );
			update_option( 'fetp_last_cron', time() - 86400 );
		} elseif ( $_POST['fetp_clean_up_frequency'] == 'off' ) {
			update_option( 'fetp_clean_up_frequency', 'off' );
		}
		
		// update theme prices
		$fetp_premium = $_POST['fetp_premium'];
		
		$premium_themes = array();
		
		if ( is_array( $fetp_premium ) && ! empty( $fetp_premium ) ) {
			
			foreach( $fetp_premium as $stylesheet ) {
				$premium_themes[] = $stylesheet;
			}
			
			foreach( $premium_themes as $theme ) {
				if ( isset( $_POST[ $theme . '_price' ] ) && ! empty( $_POST[ $theme . '_price' ] ) ) {
					update_option( $theme . '_price', $_POST[ $theme . '_price' ] );
				}
			}
			
			// remove unpriced themes
			foreach( $premium_themes as $key => $theme ) {
				if ( empty( $_POST[ $theme . '_price' ] ) ) {
					unset( $premium_theme[ $key ] );
					delete_option( $theme . '_price' );
				}
			}
		}
		
		// remove all prices if no theme is selected
		if ( empty( $premium_themes ) ) {
			$themes = wp_get_themes();
			
			foreach( $themes as $theme ) {
				delete_option( $theme->stylesheet . '_price' );
			}
		}
		
		update_option( 'fetp_premium', $premium_themes );
		
		$biz = $_POST['paypal_email'];
		$sb = $_POST['paypal_sandbox'];
		$cur = $_POST['paypal_currency'];
		$cursym = $_POST['paypal_currencysymbol'];
		$loc = $_POST['paypal_location'];
		$log = $_POST['paypal_log'];
		$subject = $_POST['paypal_subject'];
		$message = $_POST['paypal_message'];
		$htaccess = stripslashes( $_POST['fetp_htaccess'] );
		
		$sandbox = ( empty( $sb ) ) ? false : true;
		$ipn_log = ( empty( $log ) ) ? false : true;
		
		$array = array(
			"business" => "$biz",
			"sandbox" => "$sandbox",
			"currency" => "$cur",
			"cursymbol" => "$cursym",
			"location" => "$loc",
			"ipn_log" => "$ipn_log",
			"subject" => "$subject",
			"message" => "$message"
		);
		
		// Don't save this in the Database, send it straight to the file
		if ( !empty( $htaccess ) && file_get_contents( WP_CONTENT_DIR . '/themes/.htaccess' ) !== $htaccess )
			fwrite( fopen( WP_CONTENT_DIR . '/themes/.htaccess', 'w+' ), $htaccess );
		
		update_option( 'fetp_paypal_settings', $array );
		
		$fetp_options['saved'] = '<div class="updated fade"><p>Options Sucessfully <b>saved</b></p></div>';
	}
	
	// if javascript is disabled, do clean up the boring way
	if ( isset( $_GET['fetp_no_js_clean_up'] ) && is_admin() ) {
		$fetp->clean_up();
		$fetp_options['saved'] = '<div class="updated fade"><p>Folder sucessfully <b>cleaned</b></p></div>';
		header( 'Refresh:2; url=admin.php?page=fetp' );
		exit;
	}
	
	/* Get our variables after options save */
	$themes = wp_get_themes();
	$current_theme = wp_get_theme();
	$current_theme_allowed = get_option( 'fetp_enable_default_theme_download' );
	$premium = get_option( 'fetp_premium' );
	$paypal = get_option( 'fetp_paypal_settings' );
	$fetp_themes = get_option( 'fetp_themes' );
	
	// Remove current theme from queue
	if ( ! $current_theme_allowed )
		unset( $themes[ $current_theme->stylesheet ] );
?>
	<div class="fetp-options">		
		<h2>Front End Theme Preview Settings</h2>
		<?php
			if ( isset( $fetp_options['saved'] ) ) echo $fetp_options['saved'];
		?>
		<p>
			This plugin allows theme developers to add a theme preview feature to users on the front end. All you have to do is drop a shortcode or two 
			and you're good to go. More details below and in the help tab above.
		</p>
		
		<p>
			Configure the options below for further functionality.
		</p>
		
			<div class="fetp-tabs">
				<ul>
					<li class=""><a class="fetp-settings active" href="#">Settings</a></li>
					<li class=""><a class="fetp-premium" href="#">Premium Themes</a></li>
					<li class=""><a class="fetp-cart" href="#">Paypal Settings</a></li>
				</ul>
			</div>
			
			<form method="POST" id="fetp">
			<div class="fetp-content setting">
				<?php if ( isset( $_GET['fetp_ipn_preview'] ) && false !== fetp_ipn_log() ) :
					
					echo fetp_ipn_log();
					
				else: ?>
					<div class="option">
						<input type="checkbox" value="1" name="fetp_cookie" id="fetp_cookie" <?php if ( get_option( 'fetp_cookie' ) ) echo 'checked'; ?> /> :
						<label for="fetp_cookie">Allow Cookies <span class="description"> &#8212; This allows users to browse your site in preview mode</span></label>
					</div>
					
					<div class="option">
						<input type="checkbox" value="1" name="fetp_select" id="fetp_select" <?php if ( get_option( 'fetp_select' ) ) echo 'checked'; ?> /> :
						<label for="fetp_select">Add Themes Dropdown <span class="description"> &#8212; This is a list of available themes on the plugin's custom toolbar</span></label>
					</div>
					
					<div class="option">
						<input type="checkbox" value="1" name="fetp_enable_download" id="fetp_enable_download" <?php if ( get_option( 'fetp_enable_download' ) ) echo 'checked'; ?> /> :
						<label for="fetp_enable_download">Allow Theme Download <span class="description"> &#8212; This adds a 'download theme' link in the plugins toolbar</label>
					</div>
					
					<div class="option">
						<input type="checkbox" value="1" name="fetp_enable_default_theme_download" id="fetp_enable_default_theme_download" <?php if ( get_option( 'fetp_enable_default_theme_download' ) ) echo 'checked'; ?> /> :
						<label for="fetp_enable_default_theme_download">Allow Default Theme Download <span class="description"> &#8212; Allow users to download your 'default' theme</label>
					</div>
					
					<div class="option">
						<input type="checkbox" value="1" name="fetp_delete_data" id="fetp_delete_data" <?php if ( get_option( 'fetp_delete_data' ) ) echo 'checked'; ?> /> :
						<label for="fetp_delete_data">Delete Plugin Data When You Deactivate</label>
					</div>
					
					<div class="option">
						<select name="fetp_clean_up_frequency" id="fetp_clean_up_frequency">
							<option value="hourly" <?php if ( get_option( 'fetp_clean_up_frequency' ) == 'hourly' ) { ?>selected="selected"<?php } ?>>Hourly</option>
							<option value="daily" <?php if ( get_option( 'fetp_clean_up_frequency' ) == 'daily' ) { ?>selected="selected"<?php } ?>>Daily</option>
							<option value="twicedaily" <?php if ( get_option( 'fetp_clean_up_frequency' ) == 'twicedaily' ) { ?>selected="selected"<?php } ?>>Twice Daily</option>
							<option value="off" <?php if ( get_option( 'fetp_clean_up_frequency' ) == 'off' ) { ?>selected="selected"<?php } ?>>Off</option>
						</select> : 
						<label for="fetp_clean_up_frequency">
							Select When To Clear Your <code>wp-content/themes</code> of all zipped files
							<?php if ( $fetp->have_files() ) : ?>
							<a href="admin.php?page=fetp&fetp_no_js_clean_up=true" class="button-primary fetp_clean_up_now right" name="clean_up_now" id="clean_up_now"><b>Run Clean Up Now</b></a>
							<br />
							<br />
							<span class="description numfiles"><?php echo $fetp->num_files(); ?></span>
							<?php endif; ?>
						</label>
						<br />
						<div class="fetpcleaner">
							<div id="crontime" class="hidden"></div>
							<p class="timeleft"></p>
							<div class="delDone" class="hidden"></div>
						</div>
					</div>
					
					<div class="themes option last" id="fetp-themes">
						<h5>Select Themes To Add To Preview AND Download List</h5>
						<ul>
							<?php foreach( $themes as $theme ) { ?>
								<li>
									<input class="fetp-themes" type="checkbox" name="fetp_themes[]" id="<?php echo $theme->stylesheet; ?>" value="<?php echo $theme->stylesheet; ?>" <?php echo fetp_checked( $theme->stylesheet ); ?> />
									<label class="" for="<?php echo $theme->stylesheet; ?>"><?php echo $theme->Name; ?></label>
								</li>
							<?php } ?>
						</ul>
					</div>
			</div>
			
			<div class="fetp-content premium hidden" id="premium">
				<div class="themes option last">
					<h3>Check Only The Premium Themes</h3>
					<span class="description">
						This list contains themes that you have added to the preview list in the Settings Tab. 
						Check the premium theme(s) and set the corresponding price.
					</span>
					<div id="fetp-premium-themes" class="fetp-premium-themes"><?php fetp_premium_themes(); ?></div>
				</div>
			</div>
			
			<div class="fetp-content cart hidden">			
				<div class="themes option last">						
					<table class="wp-list-table widefat fixed fetp plugins">
						<thead>
						<th class="theadth">Paypal Settings</th>
						<th class="theadth"></th>
						</thead>
						
						<tr class="active">
							<th>Your Paypal Email Address</th>
							<td><input type="text" size="35" name="paypal_email" value="<?php echo _pp( 'business' ); ?>" /></td>
						</tr>
						
						<tr class="inactive">
							<th>Enable Sandbox Testing</th>
							<td><input type="checkbox" <?php if ( _pp( 'sandbox' ) ) echo 'checked="checked"'; ?> value="1" name="paypal_sandbox" />
							<br/>
							Check this option if you want to use the PayPal sandbox testing. You will need to create an account from the 
							<a href="<?php echo $fetpay->sandbox_uri(); ?>">PayPal Developer Site</a>.
							<br />
							<br />
							Remember to uncheck this when you're done testing.
							</td>
						</tr>
						
						<tr class="active">
							<th>Currency</th>
							<td>
								<input type="text" size="3" name="paypal_currency" value="<?php echo _pp( 'currency' ); ?>" /> (e.g USD, EUR, GBP, AUD)
							</td>
						</tr>
						
						<tr class="inactive">
							<th>Currency Symbol</th>
							<td>
								<input type="text" size="3" value="<?php echo _pp( 'cursymbol' ); ?>" name="paypal_currencysymbol"/> (e.g. $, &#163;, &#8364;) 
							</td>
						</tr>
						
						<tr class="active">
							<th>Your Country Code</th>
							<td>
								<input type="text" size="3" value="<?php echo _pp( 'location' ); ?>" name="paypal_location"/> (e.g. US, KE) 
							</td>
						</tr>
						
						<tr class="inactive">
							<th>Transaction Log</th>
							<td>
								<input type="checkbox" value="1" <?php if ( _pp( 'ipn_log' ) ) echo 'checked="checked"'; ?> name="paypal_log" /> 
								Check this to log IPN errors for debug. 
								<?php if ( $fetpay->ipn_log ) : ?>
								<br />
								<br />
								<a class="fetp-ipn-log button-primary" style="color: #fff!important;" href="<?php echo add_query_arg( array( 'fetp_ipn_preview' => '' ) ); ?>">View The IPN Log</a>
								<?php endif; ?>
							</td>
						</tr>
						
						<tr class="active">
							<th>Email Options</th>
							<td>
								<strong>Subject</strong>
								<br />
								<input type="text" name="paypal_subject" value="<?php echo _pp( 'subject' ); ?>" size="35" />
								<br />
								<br />
								<strong>Message</strong>
								<br />
								<textarea name="paypal_message" cols="35" rows="8"><?php echo _pp( 'message' ); ?></textarea>
								<br />
								After a successful purchase, we'll register a user (if their email is not already registered) in your site to enable them access to their themes. 
								You can specify the email subject/message here and the plugin will take care of the rest. i.e registering the user and sending them 
								the login details.
							</td>
						</tr>
						
						<tr class="inactive">
							<th>HTACCESS FILE</th>
							<td>
								<strong>DON'T edit this unless you know what you're doing.</strong>
								<br />
								<br />
								<textarea name="fetp_htaccess" cols="35" rows="8"><?php fetp_htaccess(); ?></textarea>
								<br />
									The whole point of premium themes is to make money from the sales. 
									We can't do that if the zip files are publicly accessible. This options bars direct access.
									Don't worry, peeps can still download your themes, the premium way of course :)
								<br />
								<br />
									What you need to keep in mind is that, trying to access a zip file like this, <code><?php echo site_url( '/wp-content/themes/my-theme.1.0.zip' ); ?></code> 
									won't work. All download requests need to be like <code><?php echo site_url( '/?fet_download=my-theme' ); ?></code> and/or <code><?php echo site_url( '/?get_theme=my-theme' ); ?></code> 
									for premium themes.
							</td>
						</tr>
						
					</table>
				</div>
			<?php endif; ?>
			</div>
		
		<div class="fetp-update-button left">
			
			<?php submit_button( __( 'Update Options', 'fetp' ), 'button-primary', 'fetp_save_options' ); ?>
			<span class="update"></span>
			</form>
		</div>
		
		<div class="fetp-support-project right">
			<?php echo fetp_support_project(); ?>
		</div>
	</div>
		
	<div class="clear"></div>	
<?php }

/* Scripts */
function fetp_options_js() {
global $fetp;	
	wp_enqueue_script( 'jquery' );
	wp_enqueue_style( 'fetpAdmin', plugins_url( 'css/fetp-admin.css', __FILE__ )  );
	
	if ( $fetp->have_files() )
		wp_enqueue_script( 'fetpcounter', plugins_url( 'js/jquery.countdown.js', __FILE__ ), array(), '1.0' );
}
add_action( 'admin_enqueue_scripts', 'fetp_options_js' );

/* IPN Log */
function fetp_ipn_log() {
	
	$ipn_log = ( file_exists( FETP_IPN_LOG_FILE ) ) ? file_get_contents( FETP_IPN_LOG_FILE ) : false;
	
	if ( false === $ipn_log )
		return false;
	
	if ( empty( $ipn_log ) )
		return '<div class="fetp-ipn-log" style="width: 80%; padding: 10px; border: 1px solid #999; border-radius: 5px; margin: 8px auto;">
			<span style="float: right;"><a class="button-primary" href="'.remove_query_arg( 'fetp_ipn_preview' ).'">Exit Ipn Log View</a></span>
			<p>Your IPN LOG is empty. The log is cleared whenever there\'s a theme cleanup</p>
		</div>';		
	
	$ipn_log = explode( "\n", $ipn_log );
	
	$log = '<div class="fetp-ipn-log" style="width: 80%; padding: 10px; border: 1px solid #999; border-radius: 5px; margin: 8px auto;">';
	$log .= '<span style="float: right;"><a class="button-primary" href="'.remove_query_arg( 'fetp_ipn_preview' ).'">Exit Ipn Log View</a></span>';
	foreach( $ipn_log as $line ) {
		$log .= '<p style="margin-bottom: 5px;">' . $line . '</p>';
	}
	$log .= '</div>';
	
	return $log;
}

function fetp_tabs_js() {
global $fetp;
	if ( strstr( $_SERVER['REQUEST_URI'], 'fetp' ) ) :
?>
<script type="text/javascript">
jQuery(document).ready(function($) {
	
	setTimeout( function() { $('.fade').fadeOut(2000); }, 5000 );
	
	// toggle tabs
	$("a.fetp-settings").click(function(e) {
		e.preventDefault();
		$(".fetp-tabs a").removeClass("active");
		$(".premium").hide();
		$(".cart").hide();
		$(".setting").show();		
		$(this).addClass("active");
	});
	
	$("a.fetp-premium").click(function(e) {
		e.preventDefault();
		$(".fetp-tabs a").removeClass("active");
		$(".setting").hide();
		$(".cart").hide();
		$(".premium").show();
		$(this).addClass("active");
	});
	
	$("a.fetp-cart").click(function(e) {
		e.preventDefault();
		$(".fetp-tabs a").removeClass("active");
		$(".setting").hide();
		$(".premium").hide();
		$(".cart").show();
		$(this).addClass("active");
	});
	
<?php if ( get_option( 'fetp_clean_up_frequency' ) !== 'off' && $fetp->have_files() ) : ?>
	$('#crontime').countdown({
		timestamp: (new Date()).getTime() + <?php echo $fetp->get_cron_timeleft(); ?>*1000,
		callback: function(days, hours, minutes, seconds){
			var message = 'Next clean up is <b>';
			/*message += days + " day" + ( days==1 ? '':'s' ) + ", ";*/
			message += hours + " hour" + ( hours==1 ? '':'s' ) + ", ";
			message += minutes + " minute" + ( minutes==1 ? '':'s' ) + "</b> and <b>";
			message += seconds + " second" + ( seconds==1 ? '':'s' ) + "</b> away";
			
			$("p.timeleft").html(message);
		}
	});
<?php endif; ?>
});
</script>
<?php 

	endif;
}
add_action( 'admin_footer', 'fetp_tabs_js' );

/* get paypal option */
function _pp( $option ) {
global $fetpay;
	$settings = get_option( 'fetp_paypal_settings' );
	$defaults = $fetpay->settings();
	
	return ( isset( $settings[ $option ] ) ) ? $settings[ $option ] : $defaults[ $option ];
}

/* Get premium themes */
function fetp_premium_themes( $return = false ) {
global $fetpay;	
	$fetp_themes = get_option( 'fetp_themes' );
	
	// Remove deleted themes
	foreach( $fetp_themes as $key => $value ) {
		if ( ! wp_get_theme( $value )->exists() )
			unset( $fetp_themes[ $key ] );
	}
	
	$prem = '<div class="themes option last">						
		<table class="wp-list-table widefat fixed fetp plugins">
			<thead>
				<th class="theadth">Theme</th>
				<th class="theadth">Price</th>
			</thead>';
		
		$i = 0;
		$j = 0;
		
		foreach( $fetp_themes as $theme ) {
			$class = ( $i++ % 2 == 0 ) ? 'active' : 'inactive';
			$price = get_option( $theme . '_price' );
			$checked = ( $price == '' || $price == false) ? '' : 'checked';
			$j++;
			$prem .= '<tr class="' . $class . '">
				<td> ' . $j . '. 
					<input class="fetp-premium-theme" ' . $checked . ' name="fetp_premium[]" id="' . wp_get_theme( $theme )->Name . '" type="checkbox" value="' . wp_get_theme( $theme )->stylesheet . '" />
					<label for="' . wp_get_theme( $theme )->Name . '"> Select<strong> ' . wp_get_theme( $theme )->Name . '</strong>...</label>
				</td>
				
				<td>
					<i class="fetp-price">' . $fetpay->cursymbol . '</i>
					<input type="text" name="' . $theme . '_price" id="' . $theme . '_price" value="' . $price . '" />
					<label for="' . $theme . '_price">...price it here</label>
				</td>
			</tr>';
		}
		
		$prem .= '</table>
	</div>';
	
	if ( $return ) 
		return $prem;
	else 
		echo $prem;
}


/* Premium themes checked */
function fetp_premium_checked( $stylesheet ) {
global $fetp;
	
	if ( $fetp->is_premium( $stylesheet ) )
		return 'checked="checked"';
}

/* check if theme is up for preview */
function fetp_checked( $stylesheet ) {
	$fetp_themes = get_option( 'fetp_themes' );
	
	if ( in_array( $stylesheet, $fetp_themes ) )
		return 'checked="checked"';
}

/* Get the htaccess file */
function fetp_htaccess() {
$htaccess = '<Files ~ ".zip$">
	Order allow,deny
	Deny from all
</Files>
';
	$file = WP_CONTENT_DIR . '/themes/.htaccess';
	echo ( file_exists( $file ) ) ? file_get_contents( $file ) : $htaccess;
}

/* Write the htaccess file */
function fetp_write_htaccess() {
$htaccess = '<Files ~ ".zip$">
	Order allow,deny
	Deny from all
</Files>
';
	$file = WP_CONTENT_DIR . '/themes/.htaccess';
	
	if ( ! file_exists( $file ) ) {
		fwrite( fopen( $file, 'w+' ), $htaccess );
	}
}

/* PayPal Donations */
function fetp_support_project( $link = false ) {
	$form = __( '<form action="https://www.paypal.com/cgi-bin/webscr" method="post" target="_blank">
<input type="hidden" name="cmd" value="_s-xclick">
<input type="hidden" name="hosted_button_id" value="H7MVYPXN8BUSL">
<input type="image" src="https://www.paypalobjects.com/en_US/i/btn/btn_donate_LG.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online!">
<img alt="" border="0" src="https://www.paypalobjects.com/en_US/i/scr/pixel.gif" width="1" height="1">
</form>', 'fetp' );
	
	$_link = __( 'https://www.paypal.com/cgi-bin/webscr/?cmd=_s-xclick&hosted_button_id=H7MVYPXN8BUSL', 'fetp' );
	
	if ( $link ) {
		return $_link;
	} else {
		return $form;
	}
}
ob_flush();