<?php

ob_start();

/** filename of the IPN log */
define( 'FETP_IPN_LOG_FILE', dirname(__FILE__) . '/.ipn_results.log' );

$fetp_config = fetp_get_option( 'paypal' );

$fetpay = new Fetp_PayPal_Checkout( $fetp_config );

class Fetp_PayPal_Checkout {
	
	/* Regex */
	var $regex = '|[^a-z0-9_./-]|i';
	
    /* Are we in sandbox */
	public $sandbox;
	
	/* Seller (you) Email */
    public $business;
	
	/* Transaction Currency */
    public $currency;
	
	/* Currency Symbol */
    public $cursymbol;
	
	/* Country Code */
    public $location;
	
	/* Where to go after payment */
    public $returnurl;
	
	/* What to display to Return link */
    public $returntxt;
	
	/* Where to redirect on cancel */
    public $cancelurl;
	
	/* The cart items */
    public $items = array();
	
	/* (bool) whether to log transaction details */
	public $ipn_log;
	
	/* 
	* URL That handles our IPN response 
	*
	* Doesn't seem to work anymore :( but
	* do not fret, this plugin still works
	*/
	public $notify_url;
	
	/* Paypal URL */
    const PAYPAL_HOST = 'www.paypal.com';
	
	/* Paypal Sandbox URL */
    const SANDBOX_HOST = 'www.sandbox.paypal.com';
	
	/* 
	* Mail Subject
	*
	* The buyer will be registered and emailed his login details
	* to download his/her theme
	* This specifies the mail subject
	*/
	public $subject;
	
	/*
	* Mail content
	*
	* The text to include in the email
	* 
	* The username and password are added automatically.
	* This specifies a simple thank you to the user
	*/
	public $message;

	//=======================================================//
		//==> Class constructor, with default settings that can be overridden
	//=======================================================//
	public function __construct( $config ) {
		
		$user_data = $this->user_data();
		
		// default settings
		$settings = array(
			'sandbox' => false, 
			'business' => $user_data->user_email, 
			'currency' => 'USD', 
			'cursymbol'=> '&dollar;', 
			'location' => '', 
			'returnurl'=> site_url( '/?fetp-transaction-complete' ),
			'notify_url' => site_url(),
			'returntxt'=> 'Return to ' . get_bloginfo('name'), 
			'cancelurl'=> site_url( $_SERVER['REQUEST_URI'] ), 
			'shipping' => 0,
			'ipn_log' => false,
			'subject' => 'Theme Purchase',
			'message' => sprintf( __( 'You recently purchased a theme from <strong>%s</strong>. Please use the login details in this email to access your content' ), get_bloginfo( 'name' ) ), // further details in the register_user() function below
			'items' => ( isset( $_COOKIE['fetp_items'] ) && is_array( unserialize( $_COOKIE['fetp_items'] ) ) ) ? unserialize( $_COOKIE['fetp_items'] ) : array(),
			'custom'   => '' // Custom attribute
		);

		//override default settings
		if ( !empty( $config ) ) { 
			foreach( $config as $key => $val ) {
				if ( !empty( $val ) ) { 
					$settings[$key] = $val; 
				}
			}
		}

		//Set the class attributes
		$this->sandbox  = $settings['sandbox'];
		$this->business  = $settings['business'];
		$this->currency  = $settings['currency'];
		$this->cursymbol = $settings['cursymbol'];
		$this->location  = $settings['location'];
		$this->returnurl = $settings['returnurl'];
		$this->notify_url = $settings['notify_url'];
		$this->returntxt = $settings['returntxt'];
		$this->cancelurl = $settings['cancelurl'];
		$this->shipping  = $settings['shipping'];
		$this->ipn_log  = $settings['ipn_log'];
		$this->custom    = $settings['custom'];
		$this->subject = $settings['subject'];
		$this->message = $settings['message'];
		$this->items = $settings['items'];
		
		/* 
		* Hook into the system
		*/
		
		// Add item to cart
		self::AddToCart();
		
		// Remove Item
		self::removeitem();
		
		// Verify Paypal Transaction
		self::verify();
		
		// Create plugin table
		if ( ! self::is_installed() )
			self::CreateTable();
		
		// update cart
		self::ajax_update_cart();
		
		/* clear cart contents request */
		if ( isset( $_GET['fetp-empty-cart'] ) )
			self::clear_item_session( true );
	}
	
	/* 
	* Get User Data 
	*
	* Don't freak out, I'm a nice person (most of the time). This is only to help with some default
	* values for the paypal settings in "fetp-admin.php". Once you update your info in the form, 
	* this data becomes (almost) useless.
	*/
	private function user_data() {
		require_once( ABSPATH . '/wp-includes/pluggable.php' );
		
		global $current_user; get_currentuserinfo();
		
		return $current_user;
	}
	
    //===========================//
    //= Add an item to the cart
    //===========================//
    public function addItem($item){		
		if ( ! empty( $item['quantity'] ) && is_numeric( $item['quantity'] ) && $item['quantity'] > 0 && !empty( $item['name'] ) && ! in_array( $item, $this->items ) ) { 
		
		//And add the item to the cart if it is correct
		$items = $this->items;
		$items[] = $item;
		$this->items = $items;
		}
    }


    //===============================//
		//= Add an array of items to the cart
    //===============================//
    public function addItems($items) {
        if ( ! empty( $items ) ) {
			//lopp through the items
			foreach( $items as $item ) { 
				//And add them 1 by 1
				$this->addItem( $item ); 
			}
		}
    }
	
	//====================//
		//= Remove Cart Item 
	//====================//	
	public function removeitem() {
		
		$items = $this->items;
		
		if ( isset( $_GET['fetpRem'] ) ) {
				
			foreach( $items as $item => $val ) {
				if ( $val['name'] === $_GET['item_name'] ) {
					unset( $items[$item] );
				}
			}
			
			
			if ( isset( $_COOKIE['fetp_items'] ) ) {
				
				$session_items = unserialize( $_COOKIE['fetp_items'] );
				
				foreach( $session_items as $item => $value ) {
					if ( $value['name'] === $_GET['item_name'] ) {
						unset( $session_items[ $item ] );
					}
				}
			}
			$this->items = $items;
			$this->set_item_cookie( $this->items );
			
			wp_redirect( remove_query_arg( array( 'fetpRem', 'item_name' ) ) );
			exit;
		}
	}
	
	//=====================//
		//= Add Item to existing cart
	//=====================//
	public function AddToCart() {
	global $fetp;
		
		if ( isset( $_GET['fetp-add'] ) && ! empty( $_GET['fetp-add'] ) ) {
			
			if ( ! $this->price( $_GET['fetp-add'] ) )
				return false;
			
			$stylesheet = preg_replace( $this->regex, '', $_GET['fetp-add'] );
			$theme = wp_get_theme( $stylesheet );
			
			$item = array( $stylesheet => array(
				'name' => $theme->Name,
				'id' => $stylesheet,
				'quantity' => 1,
				'price' => $this->price( $stylesheet ),
				'shipping' => 0
			));
			
			$this->addItems( $item );
		}
		
		$this->set_item_cookie( $this->items );
	}
	
	/* 
	* Ajax update cart
	*/
	public function ajax_update_cart() {
		
		/* listen for ajax request */
		if ( isset( $_GET['fetp-ajax-add-to-cart'] ) ) {
			
			/* Add Item to cart */
			$this->AddToCart();
			
			/* Response */
			echo $this->getCart();
			exit;
			
		} elseif ( isset( $_GET['fetp-ajax-empty-cart'] ) ) {
			
			/* clear session*/
			$this->clear_item_session();
			
			/* clear items */
			$this->items = array();
			
			/* Response */
			echo $this->getCart();
			exit;
			
		} elseif ( isset( $_GET['fetp-ajax-remove'] ) ) {
			
			$items = $this->items;
			
			/* Remove item */
			foreach( $items as $item => $val ) {
				if ( $val['name'] === $_GET['item_name'] ) {
					unset( $items[$item] );
				}
			}
			
			$this->set_item_cookie( $items );
			$this->items = $items;
			
			/* Response */
			echo $this->getCart();
			exit;
		}
	}
	
    //===================================//
		//=Returns a summary list of the cart content(s)
    //===================================//
    public function getCart( $hidetotal = false ) {
		if ( empty( $this->items ) && isset( $_COOKIE['fetp_items'] ) && ! empty( $_COOKIE['fetp_items'] ) )
			$this->items = unserialize( $_COOKIE['fetp_items'] );
			
        $content='<ul id="cartcontent">'; 
		$total=0; 
		$count = 0;
        $cpt=1;
		
		if ( !empty( $this->items ) ) {
			
			/* Odd/Even css classes */
			$classes = 0;
			
			foreach( $this->items as $item ) {
				
				$class = ( $classes++ % 2 == 0 ) ? 'odd' : 'even';
				
				$amount = $item['quantity']*$item['price'];
				
				$content.='<li class="cartitem '.$class.'"> <b>Theme:  </b>"'.$item['name'].'" at '.$this->cursymbol.''.$item['price'] . 
				'<p align="right" class="fetp-remove-item">
					<small><a class="fetp-remove" id="rem-' . $item['name'] . '" href="' . fetp_seturl( array( 'fetpRem' => '', 'item_name' => $item['name'] ) ) . '">Remove '.$item['name'].'</a></small>
				</p>';
				
				$content.='</li>';
				
				$total+=$amount;
				
				$count+=$item['quantity'];
				
				$cpt++;
			}	
	    }	
        
		if ( ! $hidetotal ) {
		
			if ( $total == 0 ) {
				$content.='<li class="carttotal"><span>Your Cart Is Empty</span></li>'; 
			} elseif ( $total > 0 ) {
				$emptyCart = fetp_seturl( 'fetp-empty-cart' );
				$content.='<li class="carttotal"><span>TOTAL: '.$this->cursymbol.''.$total.'</span> <span style="float: right;"> <a class="fetp-clear-cart" style="color: #FF5403!important; font-size: 13px;" href="'. $emptyCart .'">Clear Cart Contents</a></span></li>'; 
				$content.='<div class="clear"></div>';
			}
		}
		
		$content .='</ul>';
		$content .= $this->form();
		
       return $content;
    }
	
	/* Set Item Session Cookie  */
	private function set_item_cookie( $items ) {
		setcookie( 'fetp_items', serialize( $items ), time()+3600, '/' );
	}
	
	/* Clear Items Session */
	public function clear_item_session( $clear_query_args = false ) {
		setcookie( 'fetp_items', '', time()-3600, '/' );
		unset( $_COOKIE['fetp_items'] );
		
		if ( $clear_query_args ) {
			$permalink = remove_query_arg( fetp_requests() );
			wp_redirect( $permalink );
			exit;
		}
	}

    //=====================//
		//= Checkout Form
    //=====================//
    public function form(){
		
		$sandbox = '';
		if ( $this->sandbox )
			$sandbox ='sandbox.';

		$form='
			<form id="paypal_checkout" action="https://www.' . $sandbox . 'paypal.com/cgi-bin/webscr" method="POST" target="_blank">';

		//==> Variables defining a cart, there shouldn't be a need to change those <==//
		$form.='
			<input type="hidden" name="cmd" value="_cart" />
			<input type="hidden" name="upload" value="1" />			
			<input type="hidden" name="no_note" value="0" />						
			<input type="hidden" name="bn" value="PP-BuyNowBF" />					
			<input type="hidden" name="tax" value="0" />			
			<input type="hidden" name="rm" value="2" />';

		//==> Personnalised variables, they get their values from the specified settings nd the class attributes <==//
		$form.='
			<input type="hidden" name="business" value="'.$this->business.'" />
			<input type="hidden" name="handling_cart" value="'.$this->shipping.'" />
			<input type="hidden" name="currency_code" value="'.$this->currency.'" />
			<input type="hidden" name="lc" value="'.$this->location.'" />
			<input type="hidden" name="return" value="'.$this->returnurl.'" />			
			<input type="hidden" name="cbt" value="'.$this->returntxt.'" />
			<input type="hidden" name="cancel_return" value="'.$this->cancelurl.'" />
			<input type="hidden" name="notify_url" value="'.$this->notify_url.'" />
			<input type="hidden" name="custom" value="'.$this->custom.'" />';

		//==> The items of the cart <==//
		$cpt=1;
		if ( !empty( $this->items ) ) { 
			foreach( $this->items as $item ) {
				$form.='
					<div id="item_'.$cpt.'" class="itemwrap">
					<input type="hidden" name="item_name_'.$cpt.'" value="'.$item['name'].'" />
					<input type="hidden" name="quantity_'.$cpt.'" value="'.$item['quantity'].'" />
					<input type="hidden" name="item_id_'.$cpt.'" value="'.$item['id'].'" />
					<input type="hidden" name="amount_'.$cpt.'" value="'.$item['price'].'" />
					<input type="hidden" name="shipping_'.$cpt.'" value="'.$item['shipping'].'" />
					</div>';
				$cpt++;
			}
		}

		//==> The submit button <==//
		$disabled = '';
		if ( empty( $this->items ) )
			$disabled = 'disabled="disabled"';
		
		$class_disabled = '';
		if ( $disabled != '' )
			$class_disabled = 'fetp-disabled';

		$form.='
		<input style="margin-top: 5px;" id="ppcheckoutbtn" '.$disabled.' type="submit" value="Checkout With Paypal" class="fetp-button '.$class_disabled.'" />
		</form>';

		return $form;
    }
	
	// ==========================// 
		//= Paypal Sandbox URI :: nothing big
	// ==========================// 
	public function sandbox_uri() {
		return 'https://www.sandbox.paypal.com';
	}

	// ==========================//
		//= Get Theme Price
	// ==========================//
	public function price( $stylesheet ) {
		
		$premium = fetp_get_option( 'premium' );
		
		return ( isset( $premium[ $stylesheet ] ) ) ? $premium[ $stylesheet ] : '';
	}
	
	// ==========================//
		//= Paypal URL for IPN listener
	// ==========================//
    private function getPaypalHost() {
        return ( $this->sandbox ) ? self::SANDBOX_HOST : self::PAYPAL_HOST;
    }
	
	// ==========================//
		//= The IP Listener
	// ==========================//
	public function verify() {
	
	/* Run it from the front end because that's how everything should be :P */
	
	if ( ! is_admin() && isset( $_POST ) && is_array( $_POST ) ) :
		
		// read the post from PayPal system and add 'cmd'
		$req = 'cmd=_notify-validate';

		foreach ($_POST as $key => $value) {
			$value = urlencode(stripslashes($value));
			$req .= "&$key=$value";
		}

		// post back to PayPal system to validate
		$header = "POST /cgi-bin/webscr HTTP/1.0\r\n";
        $header .= "Host: ".$this->getPaypalHost()."\r\n";
		$header .= "Content-Type: application/x-www-form-urlencoded\r\n";
		$header .= "Content-Length: " . strlen($req) . "\r\n\r\n";
		$fp = @fsockopen ( 'ssl://' . $this->getPaypalHost(), "443", $errno, $errstr, 30);
		
		if ( ! $fp )  {
			return false;
		} else {		
			fputs( $fp, $header . $req );		
			while( ! feof( $fp ) ) {
				$response = fgets( $fp, 1024 );
				
				if ( strcmp( $response, "VERIFIED" ) == 0 ) {
					
					// assign posted variables to local variables
					$num_cart_items = (int)$_POST['num_cart_items'];
					$payment_status = $_POST['payment_status'];
					$payment_amount = $_POST['payment_gross'];
					$payment_currency = $_POST['mc_currency'];
					$receiver_email = $_POST['receiver_email'];
					$payer_email = $_POST['payer_email'];
					
					$style = fetp_get_theme();
					$theme = wp_get_theme( $style );
					
					/* Purchased themes */
					$themes = array();
					
					$i = 0;
					while( $i < $num_cart_items ) {
						$i++;
						$themes[] = $_POST['item_name' . $i ];
					}
					
					if ( $payment_currency == $this->currency && $payment_amount == $this->payment_gross( $themes ) && $receiver_email == $this->business ) :
						
						// Register the user with their transaction email
						$this->register_user( $payer_email );
						
						// Save the theme to user's DB info
						$this->save_transaction( $themes, $payer_email );
						
						// Success message 
						add_action( 'wp_head', array( &$this, 'transaction_complete' ), 99 );
						
						// Delete preview cookie
						fetp_delete_cookie();
						
					endif;
					
				} else if ( strcmp( $response, "INVALID" ) == 0 ) {
					
					/* stuff happened, the log will show */
					$this->debug_log( $response );
					
					/* Reset shopping cart */
					$this->clear_item_session();
					
					/* Exit preview mode */
					fetp_delete_cookie();
					
				}
			}
		fclose( $fp );
		}
	endif;
	}
	
	/* successful transaction message */
	public function transaction_complete() {
		
		$num_cart_items = ( isset( $_POST['num_cart_items'] ) ) ? (int)$_POST['num_cart_items'] : 0; 
		
		$themes = array();
		
		$i = 0;
		while( $i < $num_cart_items ) {
			$i++;
			$themes[] = $_POST['item_name' . $i ];
		}
		
		if ( empty( $themes ) )
			return false;
		
		echo '<div id="fetpay" class=""></div>
		<div class="fetpay">
			<h3>Purchase Complete<span><a href="'. remove_query_arg( 'fetp-transaction-complete' ) .'" class="exit-purchase">Close</a></span></h3>
			<p>
				Thank you for purchasing <strong>' . implode( ', ', $themes ) . '.</strong>
			</p>
			
			<p>' . $this->login_notice() . '</p>
			</div>
		</div>' . "\n";
	}
	
	/* Check theme payment gross */
	function payment_gross( $themes ) {
		$wp_themes = wp_get_themes();
		
		/* loop through themes to find purchased themes */
		$wp_theme_prices = array();
		foreach( $wp_themes as $wp_theme ) {
			
			if ( in_array( $wp_theme->Name, $themes ) )
				$wp_theme_prices[] = $this->price( $wp_theme->stylesheet );
		}
		
		/* Calculate gross themes price */
		return array_sum( $wp_theme_prices );
	}
	
	// ==============================//
		//= Function to save a successful transaction
	// ==============================//
	private function save_transaction( $wp_themes, $user_email ) {
	global $wpdb;
		
		$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . FETP_TABLE . " WHERE user_email = %s", $user_email  ) );
		
		// the user does not have prior activity
		if ( is_null( $row ) ) {
			
			$wp_all_themes = wp_get_themes();
			
			$save_themes = array();
			
			foreach( $wp_all_themes as $wp_theme ) {
				if ( in_array( $wp_theme->Name, $wp_themes ) )
					$save_themes[] = $wp_theme->stylesheet;
			}
			
			$themes = implode( ',', $save_themes );
			
			$sql = "INSERT INTO " . FETP_TABLE . " ( user_email, user_themes ) VALUES( '$user_email', '$themes' )";
			
			$wpdb->query( $sql );
			
		// the user is a regular, let's update their activity
		} elseif ( is_object( $row ) ) {
			
			$themes = explode( ' ', $row->user_themes );
			
			$wp_all_themes = wp_get_themes();
			
			$save_themes = array();
			
			foreach( $wp_all_themes as $wp_theme ) {
				if ( in_array( $wp_theme->Name, $wp_themes ) )
					$save_themes[] = $wp_theme->stylesheet;
			}
			
			/* The user has already bought this theme and for whatever reason he/she is buying it again */
			foreach( $save_themes as $key => $value ) :
				if ( $this->theme_is_saved( $value, $user_email ) ) :
					unset( $save_themes[ $key ] );
				endif;
			endforeach;
			
			$save_themes = implode( ',', array_merge( $themes, $save_themes ) );
			
			$wpdb->update( FETP_TABLE, array( 'user_themes' => $save_themes ), array( 'user_email' => $user_email ) );
		}
	}
	
	/* Check if theme exists in the user's record */
	public function theme_is_saved( $stylesheet, $user_email ) {
	global $wpdb;
		
		$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . FETP_TABLE . " WHERE user_email=%s", $user_email ) );
		
		if ( is_null( $row ) )
			return false;
		
		if ( strstr( $row->user_themes, $stylesheet ) )
			return true;
		
		return false;
	}
	
	// ====================== //
		//= Register User
		//= If the client's email is already
		//= registered, he/she will only need
		//= to log in to download
	// ====================== //
	private function register_user( $email ) {
		
		if ( ! is_email( $email ) )
			return false;
		
		// don't register if email already exists
		if ( email_exists( $email ) )
			return false;
		
		// create a username for the user
		$user = explode( '@', $email );
		$user = $user[0];
		
		// the username exists
		if ( username_exists( $user ) )
			$user = $user . '_' . mt_rand( 0, 99 ) . '_' . mt_rand( 99, 199 );
		
		// let's do another check just to be sure
		if ( username_exists( $user ) )
			$user = $user . '_' . mt_rand( 200, 299 );
		
		// password
		$pass = wp_generate_password(7);
		
		// register the user
		wp_create_user( $user, $pass, $email );
		
		// send the user his login details
		$subject = $this->subject;
		$message = $this->message;
		
		$message .= '<b>Username: </b>' . $user . '<br />';
		$message .= '<b>Password: </b>' . $pass . '<br />';
		
		$headers = 'From: ' . get_bloginfo( 'name' ) . "\r\n";
		
		/* allow developers to customize the message */
		$message = apply_filters( 'fetp_registration_message', $message );
		
		add_filter( 'wp_mail_content_type', '__set_html_content_type' );
		wp_mail( $email, $subject, $message, $headers );
		remove_filter( 'wp_mail_content_type', '__set_html_content_type' );
	}
	
	// ====================== //
		//= Text to display if the user is 
		//= logged after transaction
	// ====================== //
	private function login_notice() {
		
		if ( is_user_logged_in() ) {
			$text = __( 'See the sidebar for download link', 'fetp' );
		} else {
			$text = __( 'Please <a href="' . wp_login_url( $_SERVER['REQUEST_URI'] ) . '">login</a> to download your theme from the sidebar', 'fetp' );
		}
		
		/* Allow developers to customize this message */
		return apply_filters( 'fetp_login_notice', $text );		
	}
	
	// ====================== //
		//= Table to hold our transactions
	// ====================== //
	public function CreateTable() {
		
		$sql = "
			CREATE TABLE " . FETP_TABLE . " (
				id mediumint(9) NOT NULL AUTO_INCREMENT,
				user_email VARCHAR(100) NOT NULL,
				user_themes VARCHAR(100) NOT NULL DEFAULT '',
			UNIQUE KEY id (id))
			ENGINE = MYISAM
			CHARACTER SET utf8
			COLLATE utf8_unicode_ci;
		";
		
		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta( $sql );
	}
	
	// ==========================//
		//= Check if we have a table
	// ==========================//
	public function is_installed() {
	global $wpdb;
	
		if ( $wpdb->get_var( "SHOW TABLES LIKE '" . FETP_TABLE . "'" ) == FETP_TABLE ) {
			return true;
		} else {
			return false;
		}
	}

	// ==========================//
		//= Function to log IPN results
	// ==========================//
	private function debug_log($content) {
		if ( ! $this->ipn_log )
			return false;
			
		$content .= "\n";
		
		$date = date( 'Y, m, d H:i:s' );
		
		$content .= '<b>' . $date . '</b> ' . $content;
		
		$fp = fopen( FETP_IPN_LOG_FILE, 'a' );
		
		fwrite( $fp, $content );
		
		fclose( $fp );
	}
}
ob_flush();