<?php

/**-- Premium themes List Widget --**/

class Fetp_Download_Widget Extends WP_Widget {
	
	public function __construct() {
		parent::__construct(
	 		'fetp_download_themes',
			'Fetp Download Themes',
			array( 
				'description' => __( 'Displays the current user\'s purchased themes', 'fetp' )
			)
		);
	}
	
	/* The widget */
	public function widget( $args, $instance ) {
		extract( $args );
		
		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $before_widget;
		
		if ( ! empty( $title ) )
			echo $before_title . $title . $after_title;
			
		if ( ! is_user_logged_in() ) {
			echo '<ul><li>You must be <a href="' . wp_login_url( $_SERVER['REQUEST_URI'] ) . '">logged</a> in to view this resource</li></ul>';
		} else {
			echo $this->themes();
		}
		
		echo $after_widget;
	}
	
	public function form( $instance ) {
		if ( isset( $instance[ 'title' ] ) ) {
			$title = $instance[ 'title' ];
		}
		else {
			$title = __( 'My Themes', 'fetp' );
		}
		?>
		<p>
		<label for="<?php echo $this->get_field_name( 'title' ); ?>"><?php _e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
		</p>
		<?php 
	}
	
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		$instance['title'] = ( !empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';

		return $instance;
	}
	
	// Current user purchased themes
	private function themes() {
	global $wpdb, $current_user;
		get_currentuserinfo();
		
		$email = $current_user->user_email;
		
		$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . FETP_TABLE . " WHERE user_email = %s", $email ) );
		
		if ( is_object( $row ) && strpos( $row->user_themes, ',' ) ) {
			
			$themes = explode( ',', $row->user_themes );
			
			// A theme may have been deleted
			
			/*
			foreach( $themes as $key => $theme ) {
				if ( ! wp_get_theme( $theme )->exists() )
					unset( $themes[ $key ] );
			}
			*/
			
			echo '<ul>';
			foreach( $themes as $stylesheet ) {
				
				// check for deleted themes
				$deleted_theme = apply_filters( 'fetp_deleted_theme', '<li>The Requested theme wasn\'t found in our database. Sorry</li>' );
				
				echo ( wp_get_theme( $stylesheet )->exists() ) ? '<li>' . wp_get_theme( $stylesheet )->Name . ' &#8212; <a href="?get_theme='.$stylesheet.'">Download</a></li>' : $deleted_theme;
			}
			echo '</ul>';
			
		} elseif ( is_object( $row ) && ! strpos( $row->user_themes, ',' ) ) {
				
				$theme = $row->user_themes;
				
				// Check for deleted themes
				if ( wp_get_theme( $theme )->exists() ) {
					echo '<ul><li>' . wp_get_theme( $theme )->Name . ' &#8212; <a href="?get_theme=' . $theme . '">Download</a></li></ul>';
				} else {
					echo apply_filters( 'fetp_deleted_theme', '<ul><li>The Requested theme wasn\'t found in our database. Sorry</li></ul>' );
				}
			
		} else {
			
			echo '<ul><li>Nothing found in your theme database</li></ul>';
		}
	}
}

/**-- Theme Preview List Widget --**/

class Fetp_Preview_Widget Extends WP_Widget {
	
	public function __construct() {
		parent::__construct(
	 		'fetp_themes',
			'Fetp Themes',
			array( 
				'description' => __( 'Displays the available themes in a dropdown', 'fetp' )
			)
		);
	}	
	
	/* The widget */
	public function widget( $args, $instance ) {
	global $fetp;
		extract( $args );
		
		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $before_widget;
		
		if ( ! empty( $title ) )
			echo $before_title . $title . $after_title;
			
		echo $fetp->select_preview();
		
		echo $after_widget;
	}
	
	public function form( $instance ) {
		if ( isset( $instance[ 'title' ] ) ) {
			$title = $instance[ 'title' ];
		}
		else {
			$title = __( 'Available Themes', 'fetp' );
		}
		?>
		<p>
		<label for="<?php echo $this->get_field_name( 'title' ); ?>"><?php _e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
		</p>
		<?php 
	}
	
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		$instance['title'] = ( !empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';

		return $instance;
	}	
}

/**-- Shopping Cart Widget --**/

class Fetp_Themes_Cart_Widget Extends WP_Widget {
	
	public function __construct() {
		parent::__construct(
	 		'fetp_themes_cart',
			'Fetp Theme Paypal Shopping Cart',
			array( 
				'description' => __( 'Displays a paypal shopping cart for theme checkout', 'fetp' )
			)
		);
	}
	
	/* The widget */
	public function widget( $args, $instance ) {
	global $fetpay;
		extract( $args );
		
		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $before_widget;
		
		if ( ! empty( $title ) )
			echo $before_title . $title . $after_title;
			
		echo '<div class="fetp-checkout">' . $fetpay->getCart() . '</div>';
		
		echo $after_widget;
	}

	public function form( $instance ) {
		if ( isset( $instance[ 'title' ] ) ) {
			$title = $instance[ 'title' ];
		}
		else {
			$title = __( 'Theme Shopping Cart', 'fetp' );
		}
		?>
		<p>
		<label for="<?php echo $this->get_field_name( 'title' ); ?>"><?php _e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
		</p>
		<?php 
	}
	
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		$instance['title'] = ( !empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';

		return $instance;
	}		
}

add_action( 'widgets_init', 'fetp_widgets' );

/* Register Widgets */
function fetp_widgets() { 
	register_widget( 'Fetp_Download_Widget' ); 
	register_widget( 'Fetp_Preview_Widget' );
	register_widget( 'Fetp_Themes_Cart_Widget' );
}