<?php
/* 
* Plugin Name: Front End Theme Preview
* Plugin URI: http://upthatalley.com/
* Description: This plugins allows you to preview your themes or to allow users to preview a theme on the front end (before they can download it). It renders a different theme for the user on your site without changing the 'default' theme.
* Version: 1.2.2
* Author: Harvey J
* Author URI: http://upthatalley.wordpress.com/about
*/

ob_start();

// Kill direct access
defined( 'ABSPATH' ) || fetp_access_denied();

function fetp_access_denied() {
echo '<!DOCTYPE html>
<html>
<head>
<title>'.$_SERVER['HTTP_HOST'].' - Access Denied</title>
<style type="text/css">
	body { font: 13px Tahoma; background: #f5f5f5; }
	#deny { width: 290px; height: auto; text-align: center; margin: 150px auto; padding: 10px; border: 1px solid #dadada; border-radius: 5px; background: #fff; box-shadow: 0 10px 15px rgba(0,0,0,0.2); }
</style>
</head>
<body>
<div id="deny">
	<h2>Access Denied</h2>
	<p>Please don\'t load me like this, I\'m very shy. Thanks!</p>
</div>
</body>
</html>';
die();
}

global $wpdb;
/* Define table name */
define( 'FETP_TABLE', $wpdb->prefix . 'fetp' );

/* Load Admin Page */
require( 'class-fetp-admin.php' );

/* Theme Sales class */
require( 'fetp-transact.php' );

/* Load ziparchive extender class */
require( 'fetp-zipper.php' );

/* Load theme zip class */
require( 'class.fetp.php' );

/* Widgets */
require( 'fetp-widgets.php' );

/* Admin Page */
new Fetp_Admin();

/* Update Old Options */
if ( ! get_option( 'fetp_options' ) )
	fetp_set_new_options();

/* Get the user desired theme to preview */
function fetp_get_theme() {
	
	$regex = '|[^a-z0-9_./-]|i'; // standard $stylesheet structure
	
	$fetp_themes = fetp_get_option( 'themes' );
	
	// get $stylesheet from the url
	if ( isset( $_GET['fet_preview'] ) && !empty( $_GET['fet_preview'] ) ) {
		
		$stylesheet = preg_replace( $regex, '', $_GET['fet_preview'] );
		
		if ( wp_get_theme( $stylesheet )->exists() && in_array( $stylesheet, $fetp_themes ) ) {
			
			fetp_set_cookie( $stylesheet );
			
			return $stylesheet;			
		}
		
	// user has added an item to cart, save a cookie
	/*} elseif ( isset( $_GET['fetp-add'] ) && !empty( $_GET['fetp-add'] ) ) {
		
		$stylesheet = preg_replace( $regex, '', $_GET['fetp-add'] );
		
		if ( wp_get_theme( $stylesheet )->exists() ) {
			
			fetp_set_cookie( $stylesheet );
			
			return $stylesheet;
		}*/
		
	// get $stylesheet from a cookie
	} elseif ( fetp_cookie() ) {
		
		$stylesheet = preg_replace( $regex, '', fetp_cookie(true) );
		
		if ( wp_get_theme( $stylesheet )->exists() ) {
			return $stylesheet;
		}
		
	} else {
		
		return false;
		
	}
}

/* set cookie to enable sitewide navigation in preview mode */
function fetp_set_cookie( $stylesheet ) {
	fetp_delete_cookie(); // kill previous session
	setcookie( 'fetp_preview_session', $stylesheet, time()+60*60, '/' ); // 1 hour cookie
}

/* delete cookie to revert to 'default' theme */
function fetp_delete_cookie() {
	setcookie( 'fetp_preview_session', '', time()-360, '/' );
	unset( $_COOKIE['fetp_preview_session'] ); // just to be sure
}

/* Check/Retrieve Cookie */
function fetp_cookie( $return = false ) {
	if ( isset( $_COOKIE['fetp_preview_session'] ) ) {
		
		if ( $return ) {
			
			return $_COOKIE['fetp_preview_session'];
			
		} else {
			
			return true;
			
		}
		
	} else {
		
		return false;
		
	}
}

/* select $stylsheet from cookie *
function fetp_get_cookie_stylesheet() {
	$cookie = fetp_cookie( true );
	
	return fetp_get_theme( $cookie );
}*/

/* prepare url */
function fetp_seturl( $arg = '', $value = '' ) {
	
	// remove existing requests from url
	$uri = remove_query_arg( fetp_requests() );
	
	if ( ! is_array( $arg ) )
		return add_query_arg( array( $arg => $value ), $uri );
	
	// add our query
	return add_query_arg( $arg, $uri );
}

/* Preview Set Up */
function fetp_setup_theme() {

	// get $stylesheet
	$stylesheet = fetp_get_theme();
	
	$fetp_themes = fetp_get_option( 'themes' );
	
	if ( ! in_array( $stylesheet, $fetp_themes ) || empty( $stylesheet ) )
		return false;
	
	// preview
	if ( ! is_admin() ) {
		
		// Load Selected Theme
		add_filter( 'stylesheet', 'fetp_get_theme' );
		add_filter( 'template', 'fetp_get_theme' );
		
		// toggler
		add_action( 'wp_head', 'fetp_bar' );		
	}
}
add_action( 'setup_theme', 'fetp_setup_theme' );

/*
* Preview Toolbar
*/
function fetp_bar() {
global $fetp, $fetpay;
	
	$theme = wp_get_theme( fetp_get_theme() );
	
?>
	<div id="fetp-bar">
	<div class="fetp-bar-wrap">
		
		<div class="fetp-control">
			<a href="<?php echo fetp_exit(); ?>" class="fetp-button left exit" title="Exit Preview Mode">Close Preview</a>
			
			<?php 
				if ( $theme->Name !== get_option( 'current_theme' ) && current_user_can( 'switch_themes' ) ) : ?>
					<a style="position: relative; right: 14px;" href="<?php echo fetp_seturl( 'fet_switcher', $theme->stylesheet ); ?>" class="fetp-button switcher right">Activate <?php echo $theme->Name; ?></a>
			<?php endif; ?>
			<div class="clear"></div>
		</div>
		
		<a href="#" class="fetp-title one current">Description<span class="fetp-arrow right toggle">&nbsp;</span></a>
		<div class="details-tab tab-one">
			<p align="left" class="previewing">Previewing: <?php echo $theme->Name; ?></p>
			<a title="<?php echo $theme->Name . ' ' . $theme->Version; ?>" href="<?php echo content_url( '/themes/' . fetp_get_theme() . '/' . $theme->screenshot ); ?>">
				<img class="screenshot" src="<?php echo content_url( '/themes/' . fetp_get_theme() . '/' . $theme->screenshot ); ?>" />
			</a>
			<p class="description">
				<?php echo fetp_excerpt( $theme->Description, 0, 280, '...<a href="'.$theme->get('ThemeURI').'"> &raquo; See More</a>' ) ?>
			</p>
		</div>
		
		<a href="#" class="fetp-title two">Options<span class="fetp-arrow right">&nbsp;</span></a>
		<div class="details-tab tab-two fetp-hidden">
			<?php do_action( 'fetp_action' ); ?>
		</div>
		
		<a href="#" class="fetp-title three">Cart<span class="fetp-arrow right">&nbsp;</span></a>
		<div id="fetp-cart-tab" class="details-tab tab-three fetp-hidden cart">
			<?php echo $fetpay->getCart(); ?>
		</div>
	</div>
	</div>
	
	<div id="fetp-collapse" class="fetp-collapse-wrap">
		<div class="fetp-collapse"></div>
		<span class="fetp-collapse-label"><a href="#">Collapse Panel</a></span>
	</div>
<?php	
}

/* Check if we're in preview */
function fetp_is_preview() {
	if ( fetp_get_theme() )
		return true;
	
	return false;
}

/*
* Hide the admin bar when in preview
*/
function fetp_hide_admin_bar() {
	if ( fetp_is_preview() ) :
		
		// this removes the html
		add_filter( 'show_admin_bar', '__return_false' );
		
		// this removes the css that pushes down the content
		remove_action( 'wp_head', '_admin_bar_bump_cb' );
	endif;
}
add_action( 'wp', 'fetp_hide_admin_bar', 999 );

/* Is Theme Free/Premium */
function fetp_theme_status( $stylesheet ) {
global $fetp;
		
	$data = wp_get_theme( $stylesheet );
		
	if ( $fetp->is_premium( $stylesheet ) ) {
		$link = fetp_buy_url( $stylesheet );
	} else {
		$link = '<br /><br /><a class="fetp-button" href="' . fetp_seturl( 'fet_download', $stylesheet ) . '" title="Download '.$data->Name.'">Download</a>';
	}
	
	return $link;
}

/* Purchase link */
function fetp_buy_url( $stylesheet, $url = false ) {
global $fetpay;

	$uri = fetp_seturl( 'fetp-add', $stylesheet );
	
	if ( $url )
		return $uri;
	else	
		return '<br /><br /><a href="'.$uri.'" class="fetp-button add-to-cart">' . $fetpay->cursymbol . $fetpay->price( $stylesheet ) . ' Add To Cart</a><span class="fetp-loading fetp-hidden">Updating cart...</span>';
}

/* 
* Theme description excerpt
*
* Some themes have lengthy descriptions. We don't want that. Overflow issues.
* @param $str - (string) Theme Description
* @param $start - (int) where in $str to start excerpt grab
* @param $min - (int) Minimum $str length that can be shortened
* @param $hellip - (bool|string) What to display after filtering out the excess content. 'false' for '...'
*/
function fetp_excerpt( $str, $start = 0, $min = 100, $hellip = false ) {
	if ( strlen( $str ) > $min ) {
		$excerpt = substr( $str, $start, $min-3 );
		
		$lastspace = strrpos( $excerpt, ' ' );
		
		$excerpt = substr( $excerpt, 0, $lastspace );
		
		if ( $hellip ) {
			
			$excerpt .= $hellip; 
		} else {
			
			$excerpt .= '...';
		}
	} else {
		
		$excerpt = $str;
	}
return $excerpt;
}

/* Add Download Link */
function fetp_add_download_link( $fetp ) {
	require_once( ABSPATH . '/wp-includes/pluggable.php' );
	
	$current_theme = get_option( 'current_theme' );
	
	$stylesheet = fetp_get_theme();
	
	$data = wp_get_theme( $stylesheet );
	
	if ( fetp_get_option( 'enable_download' ) ) {
		$fetp .= fetp_theme_status( $stylesheet );
		$fetp .= '<p>Version: '. $data->get('Version') . '</p>';
	}
	
	echo $fetp;
}
add_action( 'fetp_action', 'fetp_add_download_link' );

/* Theme Selector On FETP Toolbar */
function fetp_toolbar_select( $select ) {
	if ( ! fetp_get_option( 'select' ) )
		return false;

	$fetp_themes = fetp_get_option( 'themes' );
	
	/* Remove deleted themes */
	foreach( $fetp_themes as $key => $value ) {
		if ( ! wp_get_theme( $value )->exists() )
			unset( $fetp_themes[ $key ] );
	}
	
	$select .= '<p>Preview Another Theme:</p>';
	$select .= '<select class="fetp-select-preview" name="fet_preview" onchange="window.location.href=this.value"><option value="#"> -- Select Theme -- </option>';
		foreach ( $fetp_themes as $fetp_theme ) {
			$select .= '<option '.fetp_selected( $fetp_theme ).' value="'. fetp_seturl( 'fet_preview', $fetp_theme ) .'">'. wp_get_theme( $fetp_theme )->Name .'</option>';
		}
	$select .='</select>';
	echo $select;
}
add_action( 'fetp_action', 'fetp_toolbar_select', 1 );

function fetp_selected($stylesheet) {
	if ( fetp_get_theme() == $stylesheet ) {
		return 'selected';
	}
}

function fetp_scripts() {	
global $fetp;
	wp_enqueue_script( 'jquery' );
	wp_enqueue_script( 'fetp',  plugins_url( 'js/fetp.js', __FILE__ ) );
	( fetp_is_preview() ) ? wp_enqueue_style( 'fetpcss', plugins_url( 'css/fetp.css', __FILE__ ) ) : '';
}
add_action( 'wp_enqueue_scripts', 'fetp_scripts' );

/* Transaction notice css */
( isset( $_GET['fetp-transaction-complete'] ) ) ? add_action( 'wp_head', 'transaction_complete_css', 98 ) : '';
function transaction_complete_css() { ?>
<style type="text/css">
	#fetpay {
		width: 100%;
		height: 100%;
		position: fixed;
		top: 0;
		left: 0;
		z-index: 16000;
		background: #000;
		background: rgba( 0, 0, 0, 0.5 );
	}

	.fetpay {
		width: 400px;
		min-height: 80px;
		background: #f5f5f5;
		border: 1px solid silver;
		border-radius: 5px;
		box-shadow: 0 4px 8px rgba(0,0,0,0.6);
		margin: 150px 330px;
		position: fixed;
		top: 0;
		left: 0;	
		z-index: 18500;
	}

	.fetpay p {
		padding: 5px 8px;
		margin: 0;
	}

	.fetpay h3 {
		font: normal 18px Georgia;
		text-align: center;
		padding: 5px 0;
		margin: 0;
		border-bottom: 1px solid #ccc;
	}

	.fetpay h3 span {
		float: right;
		font-size: 11px;
		padding: 5px 6px;
	}

	.fetpay h3 span a {
		color: #777;
		text-decoration: none;
	}
</style>
<?php
}

/* Exit Preview URL */
function fetp_exit() {
	
	$exit = remove_query_arg( fetp_requests() );
	
	$exit = add_query_arg( array( 'fetp_exit' => '' ), $exit );
	
	return $exit;
}

/*
* Plugin URI requests 
*
* An array of all requests by this plugin
*/
function fetp_requests() {
	return array(
		"fetp_exit",
		"fet_switcher",
		"fetp-add",
		"fetpRem",
		"item_name",
		"fet_preview",
		"fetp-ajax-add-to-cart",
		"fetp-empty-cart",
		"fetp-transaction-complete",
		"fet_download"
	);
}

/* 
* Reset cookie and exit preview 
*/
function fetp_exit_preview() {
	$uri = $_SERVER['REQUEST_URI'];
	
	if ( isset( $_GET['fetp_exit'] ) ) {
		fetp_delete_cookie();
		wp_redirect( remove_query_arg( fetp_requests(), $uri ) );
		exit;
	}
}
add_action( 'wp', 'fetp_exit_preview' );

/* Activation Hook */
function fetp_activate() {	
	$options = array(
		'cookie' => '1',
		'themes' => array(),
		'last_cron' => time(),
		'clean_up_frequency' => 'off'
	);
	
	if ( ! get_option( 'fetp_options' ) )
		update_option( 'fetp_options', $options );
}
register_activation_hook( __FILE__, 'fetp_activate' );

/* Deactivation Hook */
function fetp_deactivate() {	
	if ( fetp_get_option( 'delete_data' ) == '1' )
		delete_option(  'fetp_options' );
}
register_deactivation_hook( __FILE__, 'fetp_deactivate' );

/* Plugin Links */
function fetp_plugin_action_links( $links, $file ) {

	if ( $file == plugin_basename( __FILE__ ) ) {
		$links[] = '<a href="' . admin_url( 'admin.php?page=fetp' ) . '">'.__( 'Settings', 'fetp' ).'</a>';
	}
	return $links;
}
add_filter( 'plugin_action_links', 'fetp_plugin_action_links', 10, 2 );

function fetp_plugin_meta( $meta, $file ) {
	
	if ( $file == plugin_basename( __FILE__ ) ) {
		$meta[] = '<a href="' . admin_url( 'admin.php?page=fetp' ) . '">'.__( 'Settings', 'fetp' ).'</a>';
		$meta[] = '<a href="'.fetp_support_project(true).'" target="_blank">'.__( 'Donate', 'fetp' ).'</a>';
	}
	
	return $meta;
}
add_filter( 'plugin_row_meta', 'fetp_plugin_meta', 10, 2 );

/* Plugin Info */
function fetp_plugin_info() {
require_once( ABSPATH . '/wp-admin/includes/plugin.php' );

	$fetp_info =  get_plugin_data( dirname( __FILE__ ) . '/fetp.php' );
	return $fetp_info;
}

/* Debug Only */
function fetp_dump( $bug = '' ) {	
	echo '<pre>';
	var_dump( $bug );
	echo '</pre>';
}
ob_flush();